*******************************************************************************************
*  sublibrary to picasso96
*******************************************************************************************

	MACHINE	68020

	NOLIST

	include	lvo/exec.i
	include	exec/libraries.i
	include	exec/resident.i
	include	exec/initializers.i
	include	exec/alerts.i
	include	exec/ables.i
	include	graphics/rastport.i

	include	boardinfo.i
	include	settings.i

	include	macros.i
	include	CirrusGD542X.i

	include	CirrusGD542X.chip_rev.i

	LIST

NAME		MACRO
		CIRRUSNAME
		ENDM

;-----------------------------------------------

Start:						; just in case we are executed
	moveq	#-1,d0
	rts

;-----------------------------------------------

RomTagPri	EQU	0

RomTag:
	dc.w	RTC_MATCHWORD
	dc.l	RomTag
	dc.l	EndCode
	dc.b	RTF_AUTOINIT
	dc.b	VERSION
	dc.b	NT_LIBRARY
	dc.b	RomTagPri
	dc.l	LibName
	dc.l	IDString
	dc.l	InitTable

;-----------------------------------------------

LibName:		NAME
		dc.b	'.chip',0
IDString:		VSTRING
VERString:		VERSTAG
		even

;-----------------------------------------------

InitTable:
	dc.l	chip_SIZEOF			; size of library base data space (LIB_POSSIZE)
	dc.l	funcTable			; pointer to function initializers
	dc.l	dataTable			; pointer to data initializers
	dc.l	initRoutine			; routine to run

;-----------------------------------------------

funcTable:
	dc.l	Open
	dc.l	Close
	dc.l	Expunge
	dc.l	Null

	dc.l	InitGC				; the only externaly visible function!

	dc.l	-1

;-----------------------------------------------

dataTable:
	INITBYTE	LN_TYPE,NT_LIBRARY
	INITBYTE	LN_PRI,-60		; no one looks for it, so put it behind the rest
	INITLONG	LN_NAME,LibName
	INITBYTE	LIB_FLAGS,LIBF_SUMUSED!LIBF_CHANGED
	INITWORD	LIB_VERSION,VERSION
	INITWORD	LIB_REVISION,REVISION
	INITLONG	LIB_IDSTRING,IDString
	dc.l		0

;-----------------------------------------------

initRoutine:					; d0.l: library base, a0: segment list

	move.l	d0,a1
	move.l	a6,chip_ExecBase(a1)
	move.l	a0,chip_SegmentList(a1)
	rts

*****************************************************************
Open:
* a6:	library
* d0:	version
*****************************************************************
	addq.w	#1,LIB_OPENCNT(a6)		; mark us as having another opener
	bclr	#LIBB_DELEXP,chip_Flags(a6)	; prevent delayed expunges
	move.l	a6,d0
	rts

*****************************************************************
Close:
* a6:	library
*****************************************************************
	moveq	#0,d0				; set the return value
	subq.w	#1,LIB_OPENCNT(a6)		; mark us as having one fewer openers
	bne	.end				; see if there is anyone left with us open

	btst	#LIBB_DELEXP,chip_Flags(a6)	; see if we have a delayed expunge pending
	beq	.end
	bsr	Expunge
.end:
	rts

*****************************************************************
Expunge:
* a6:	library
*****************************************************************
	PUSH	d2/a5/a6
	move.l	a6,a5
	move.l	chip_ExecBase(a5),a6

	tst.w	LIB_OPENCNT(a5)			; see if anyone has us open
	beq	.not_open

	bset	#LIBB_DELEXP,chip_Flags(a5)	; it is still open.  set the delayed expunge flag
	moveq	#0,d0
	bra	.end

.not_open:
	move.l	chip_SegmentList(a5),d2		; go ahead and get rid of us.  Store our seglist in d2

	move.l	a5,a1				; unlink from library list
	CALL	Remove

	moveq	#0,d0				; free our memory
	move.l	a5,a1
	move.w	LIB_NEGSIZE(a5),d0
	sub.l	d0,a1
	add.w	LIB_POSSIZE(a5),d0
	CALL	FreeMem

	move.l	d2,d0				; set up our return value

.end:
	POP	d2/a5/a6
	rts

*****************************************************************
Null:
*****************************************************************
	moveq	#0,d0
	rts

*****************************************************************
InitGC:
* a0:	struct BoardInfo
*****************************************************************
	PUSH	a2
	move.l	a0,a2

; first, init type and chip dependent flags
	move.l	#GCT_CirrusGD542x,gbi_GraphicsControllerType(a2)
	move.l	#PCT_CirrusGD542x,gbi_PaletteChipType(a2)
	ori.l	#BIF_NOMEMORYMODEMIX,gbi_Flags(a2)
	ori.w	#RGBFF_PLANAR|RGBFF_CHUNKY|RGBFF_B8G8R8|RGBFF_R5G6B5PC|RGBFF_R5G5B5PC,gbi_RGBFormats(a2)	; supported RGBFormats
	ori.w	#RGBFF_B8G8R8|RGBFF_R8G8B8,gbi_SoftSpriteFlags(a2)

; now, write pointers to the chip functions into boardinfo
; SetGC uses modeinfo to set display timing
	lea	SetGC(pc),a1
	move.l	a1,gbi_SetGC(a2)		; mandatory
; SetPanning sets view origin e.g. for overscanned displays
	lea	SetPanning(pc),a1
	move.l	a1,gbi_SetPanning(a2)		; mandatory
; CalculateBytesPerRow calculates how many bytes a display row for a given mode has
	lea	CalculateBytesPerRow(pc),a1
	move.l	a1,gbi_CalculateBytesPerRow(a2)		; mandatory
; CalculateMemory calculates the physical memory address to a given logical address
	lea	CalculateMemory(pc),a1
	move.l	a1,gbi_CalculateMemory(a2)	; mandatory
; GetCompatibleFormats returns a mask of compatible RGBFormats to a supplied RGBFormat
	lea	GetCompatibleFormats(pc),a1
	move.l	a1,gbi_GetCompatibleFormats(a2)	; mandatory

; Cirrus has internal DAC, so put this function here
; SetDAC sets DAC mode (CLUT, HiColor15 etc)
	lea	SetDAC(pc),a1
	move.l	a1,gbi_SetDAC(a2)		; mandatory, may be overwritten by board
; SetColorArray sets colors from boardinfo->CLUT
	lea	SetColorArray(pc),a1
	move.l	a1,gbi_SetColorArray(a2)	; mandatory
; SetDPMSLevel sets degrees of power management
	lea	SetDPMSLevel(pc),a1
	move.l	a1,gbi_SetDPMSLevel(a2)		; optional, useful if chip offers it

; misc standard VGA features
	lea	SetDisplay(pc),a1
	move.l	a1,gbi_SetDisplay(a2)		; mandatory
	lea	SetMemoryMode(pc),a1
	move.l	a1,gbi_SetMemoryMode(a2)	; mandatory
	lea	SetWriteMask(pc),a1
	move.l	a1,gbi_SetWriteMask(a2)		; mandatory
	lea	SetReadPlane(pc),a1
	move.l	a1,gbi_SetReadPlane(a2)		; mandatory
	lea	SetClearMask(pc),a1
	move.l	a1,gbi_SetClearMask(a2)		; mandatory
	lea	WaitVerticalSync(pc),a1
	move.l	a1,gbi_WaitVerticalSync(a2)	; mandatory

; clock functions
	lea	SetClock(pc),a1
	move.l	a1,gbi_SetClock(a2)
	lea	ResolvePixelClock(pc),a1
	move.l	a1,gbi_ResolvePixelClock(a2)
	lea	GetPixelClock(pc),a1
	move.l	a1,gbi_GetPixelClock(a2)

	move.l	(gbi_Flags,a2),d0
	btst	#BIB_OVERCLOCK,d0
	beq	.normal
	move.l	#NUMCLOCKS+OVERCLOCKS,d0
	bra	.overclock
.normal:
	move.l	#NUMCLOCKS,d0
.overclock:
	move.l	d0,gbi_PixelClockCount+PLANAR*4(a2)
	move.l	d0,gbi_PixelClockCount+CHUNKY*4(a2)
	move.l	d0,gbi_PixelClockCount+TRUECOLOR*4(a2)
	move.l	(gbi_Flags,a2),d0
	btst	#BIB_OVERCLOCK,d0
	beq	.normalhi
	move.l	#NUMHICOLORCLOCKS+OVERCLOCKS,d0
	bra	.overclockhi
.normalhi:
	move.l	#NUMHICOLORCLOCKS,d0
.overclockhi:
	move.l	d0,gbi_PixelClockCount+HICOLOR*4(a2)

	move.w	#6,gbi_BitsPerCannon(a2)

; Cirrus 542x has 9 bits for horizontal timings
	move.w	#(1<<9)-1,d0
	lsl.w	#3,d0
	move.w	d0,gbi_MaxHorValue+PLANAR*2(a2)
	move.w	d0,gbi_MaxHorValue+CHUNKY*2(a2)
	move.w	d0,gbi_MaxHorValue+HICOLOR*2(a2)
	move.w	d0,gbi_MaxHorValue+TRUECOLOR*2(a2)
; Cirrus 542x has 10 bits for vertical timings (+ DoubleVertical)
	move.w	#(1<<11)-1,d0
	move.w	d0,gbi_MaxVerValue+PLANAR*2(a2)
	move.w	d0,gbi_MaxVerValue+CHUNKY*2(a2)
	move.w	d0,gbi_MaxVerValue+HICOLOR*2(a2)
	move.w	d0,gbi_MaxVerValue+TRUECOLOR*2(a2)
; maximum allowed BitMap sizes. Fill out all depths made available in gbi_RGBFormats
	move.w	#1600,gbi_MaxHorResolution+PLANAR*2(a2)
	move.w	#1600,gbi_MaxVerResolution+PLANAR*2(a2)
	move.w	#1600,gbi_MaxHorResolution+CHUNKY*2(a2)
	move.w	#1600,gbi_MaxVerResolution+CHUNKY*2(a2)
	move.w	#1280,gbi_MaxHorResolution+HICOLOR*2(a2)
	move.w	#1280,gbi_MaxVerResolution+HICOLOR*2(a2)
	move.w	#800,gbi_MaxHorResolution+TRUECOLOR*2(a2)
	move.w	#800,gbi_MaxVerResolution+TRUECOLOR*2(a2)

; now finally init the chip to something like 640x480x8 in about 31 kHz
; consult your chip manufacturer's manual
	move.l	gbi_Flags(a2),d0
	move.l	gbi_RegisterBase(a2),a0

; wake up
	move.b	#$16,$46E8(a0)
	move.b	#1,$102(a0)
	move.b	#$0e,$46E8(a0)

	move.b	#1,VIDEOSYSENABLE(a0)

	move.b	#$ff,DACD(a0)			; PixelMask

	move.b	#$6f,MISCOUTPUTW(a0)

	SetTS	#$12,TS_UnlockAllExtensions
	SetTS	#$01,TS_TSMode
	SetTS	#$b0,TS_DRAMControl
	SetTS	#$03,TS_SynchronousReset	; restart sequencer
	SetTS	#$ff,TS_WritePlaneMask
	SetTS	#$00,TS_FontSelect
	SetTS	#$0e,TS_MemoryMode
	SetTS	#$80,TS_EEPROMControl
	SetTS	#$d4,TS_PerformanceTuning	; was $dd / good: $d4
	SetTS	#$02,TS_SignatureGeneratorControl
	SetTS	#$65,TS_VCLK3Numerator
	SetTS	#$3b,TS_VCLK3Denominator

	SetTS	#$04,TS_GraphicsCursorAttributes

	SetTS	#60,TS_GraphicsCursorPattern
	SetTS	#$1c,TS_MCLKSelect

	SetCRTC	#$5f,CRTC_HorizontalTotal
	SetCRTC	#$4f,CRTC_HorizontalDisplayEnd
	SetCRTC	#$50,CRTC_HorizontalBlankStart
	SetCRTC	#$82,CRTC_HorizontalBlankEnd
	SetCRTC	#$54,CRTC_HorizontalSyncStart
	SetCRTC	#$80,CRTC_HorizontalSyncEnd
	SetCRTC	#$bf,CRTC_VerticalTotal
	SetCRTC	#$1f,CRTC_OverflowLow
	SetCRTC	#$00,CRTC_PresetRowScan
	SetCRTC	#$40,CRTC_MaximumRowAddress
	SetCRTC	#$00,CRTC_CursorStartRowAddress
	SetCRTC	#$00,CRTC_CursorEndRowAddress
	SetCRTC	#$00,CRTC_LinearStartingAddressMiddle
	SetCRTC	#$00,CRTC_LinearStartingAddressLow
	SetCRTC	#$00,CRTC_CursorAddressMiddle
	SetCRTC	#$00,CRTC_CursorAddressLow
	SetCRTC	#$9c,CRTC_VerticalSyncStart
	SetCRTC	#$0e,CRTC_VerticalSyncEnd
	SetCRTC	#$8f,CRTC_VerticalDisplayEnd
	SetCRTC	#$50,CRTC_RowOffset
	SetCRTC	#$00,CRTC_UnderlineRowAddress
	SetCRTC	#$96,CRTC_VerticalBlankStart
	SetCRTC	#$b9,CRTC_VerticalBlankEnd
	SetCRTC	#$c3,CRTC_CRTCMode
	SetCRTC	#$ff,CRTC_LineCompare
	SetCRTC	#$00,CRTC_InterlaceEnd
	SetCRTC	#$00,CRTC_MiscellaneousControl
	SetCRTC	#$82,CRTC_ExtendedDisplayControls

	SetGDC	#$00,GDC_SetReset
	SetGDC	#$00,GDC_EnableSetReset
	SetGDC	#$00,GDC_ColorCompare
	SetGDC	#$00,GDC_DataRotateFunctionSelect
	SetGDC	#$00,GDC_ReadPlaneSelect
	SetGDC	#$00,GDC_GDCMode
	SetGDC	#$00,GDC_Offset0
	SetGDC	#$ec,GDC_Offset1
	SetGDC	#$01,GDC_Miscellaneous
	SetGDC	#$0f,GDC_ColorCare
	SetGDC	#$ff,GDC_BitMask
	SetGDC	#$28,GDC_GDCModeExtensions

; fill attribute controller's registers
	ResetATC

	moveq	#0,d0
.ATC_PaletteRAM_loop:
	move.b	d0,ATCI(a0)
	move.b	d0,ATCD(a0)
	addq.w	#1,d0
	cmp.b	#$f,d0
	ble	.ATC_PaletteRAM_loop

	ResetATC

	SetATC	#$01,ATC_ModeControl
	SetATC	#$00,ATC_OverscanColor
	SetATC	#$0f,ATC_ColorPlaneEnable
	SetATC	#$00,ATC_HorizontalPixelPanning
	SetATC	#$00,ATC_ColorSelect

	move.b	#CRTC_ID,(CRTCI,a0)
	move.b	(CRTCD,a0),d0
	and.b	#CL_GD54xx,d0
	move.b	d0,(gbi_ChipData,a2)		; keep type for sprite etc.
	cmp.b	#CL_GD5429,d0
	beq	.has_blitter_and_hardwaresprite
	cmp.b	#CL_GD5428,d0
	beq	.has_blitter_and_hardwaresprite
	cmp.b	#CL_GD5426,d0
	beq	.has_blitter_and_hardwaresprite
	cmp.b	#CL_GD5424,d0
	beq	.has_hardwaresprite
	cmp.b	#CL_GD5422,d0
	beq	.has_hardwaresprite
	bra	.unknown_542x

.has_blitter_and_hardwaresprite:

; WaitBlitter waits until blitter is ready
	movea.l	gbi_ExecBase(a2),a1
	btst	#AFB_68040,AttnFlags+1(a1)
	bne	.goodcpu
	btst	#AFB_68030,AttnFlags+1(a1)
	beq	.goodcpu
; must be 030
	lea	WaitBlitter030(pc),a1
	bra	.continue
.goodcpu:
	lea	WaitBlitter(pc),a1
.continue:
	move.l	a1,gbi_WaitBlitter(a2)		; mandatory

; blitter functions (optional)
; BlitRect uses blitter to move rectangle
	lea	BlitRect(pc),a1
	move.l	a1,gbi_BlitRect(a2)		; optional
; InvertRect uses blitter to invert rectangle
	lea	InvertRect(pc),a1
	move.l	a1,gbi_InvertRect(a2)		; optional
; FillRect uses blitter to fill rectangle
	lea	FillRect(pc),a1
	move.l	a1,gbi_FillRect(a2)		; optional
; BlitPattern uses blitter to color expand a b/w pattern repeatedly
	lea	BlitPattern(pc),a1
	move.l	a1,gbi_BlitPattern(a2)		; optional
; BlitTemplate uses blitter to color expand bit plane for Text() and Area...()
	lea	BlitTemplate(pc),a1
	move.l	a1,gbi_BlitTemplate(a2)		; optional
; BlitPlanar2Chunky uses blitter to blit a rectangle of a planar bitmap into a chunky one
	lea	BlitPlanar2Chunky(pc),a1
	move.l	a1,gbi_BlitPlanar2Chunky(a2)	; optional
; BlitRectNoMaskComplete uses blitter to blit source to destination without mask with any minterm
	lea	BlitRectNoMaskComplete(pc),a1
	move.l	a1,gbi_BlitRectNoMaskComplete(a2)	; optional


	SetGDC	#$04,GDC_BLTStartStatus		; Reset BLT engine
	ori.l	#BIF_BLITTER,gbi_Flags(a2)

.has_hardwaresprite:

; hardware sprite functions
; SetSprite activates hardware sprite
	lea	SetSprite(pc),a1
	move.l	a1,gbi_SetSprite(a2)		; mandatory when offering hardware sprite
; SetSpritePosition sets hardware sprite position
	lea	SetSpritePosition(pc),a1
	move.l	a1,gbi_SetSpritePosition(a2)	; mandatory when offering hardware sprite
; SetSpriteImage sets hardware sprite image
	lea	SetSpriteImage(pc),a1
	move.l	a1,gbi_SetSpriteImage(a2)	; mandatory when offering hardware sprite
; SetSpriteColor sets hardware sprite colors
	lea	SetSpriteColor(pc),a1
	move.l	a1,gbi_SetSpriteColor(a2)	; mandatory when offering hardware sprite

	ori.l	#BIF_HARDWARESPRITE|BIF_DBLSCANDBLSPRITEY,gbi_Flags(a2)

.unknown_542x:
	POP	a2
	rts

***********************************************************

TrueColorMClks	dc.b	$1c,$1c,$1c,$1c,$1c,$1c,$1e,$1e
HiColorMClks	dc.b	$1c,$1c,$1c,$1c,$20,$1c,$1e,$1e
ClutMClks	dc.b	$1c,$1c,$1c,$1c,$1c,$1c,$1e,$1e

***********************************************************
SetGC:
* set mode according to ModeInfo
***********************************************************
* a0: 	struct BoardInfo
* a1: 	struct ModeInfo
* d0:	BOOL Border
***********************************************************
	PUSH	d2-d6

	move.l	a1,gbi_ModeInfo(a0)
	move.w	d0,d4
	move.w	d0,gbi_Border(a0)
	move.l	gbi_RegisterBase(a0),a0

	moveq	#0,d6

	moveq	#0,d5
	move.w	gmi_HorTotal(a1),d5
	move.w	gmi_Width(a1),d3
	move.b	gmi_Flags(a1),d2

	move.b	#CRTC_CRTCMode,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#~(1<<2),d0

; Cirrus needs this for modes higher than 1024 lines
	and.b	#~GMF_DOUBLEVERTICAL,d2
	btst	#GMB_INTERLACE,d2
	bne	.interlace
	cmp.w	#1024,gmi_VerTotal(a1)
	blt	.vertical_value_ok
	or.b	#GMF_DOUBLEVERTICAL,d2
.double_vertical:
	or.b	#(1<<2),d0				; Multiply Vertical Registers by Two
.interlace:
	moveq	#1,d6
.vertical_value_ok:
	move.b	d0,CRTCD(a0)
	move.b	d2,gmi_Flags(a1)

	move.b	gmi_Depth(a1),d1
	cmp.b	#4,d1
	ble	.pixel_nibble
	cmp.b	#8,d1
	ble	.pixel_byte
	cmp.b	#16,d1
	ble	.pixel_word

; 24 bit mode adjustments
.pixel_triplebyte:
	move.b	#CRTC_ID,(CRTCI,a0)
	move.b	(CRTCD,a0),d0
	lsr.w	#2,d0
	and.w	#%111,d0
	move.b	(TrueColorMClks,pc,d0.w),d0
	SetTS	d0,TS_MCLKSelect

	move.b	#TS_ExtendedSequencerMode,TSI(a0)
	or.b	#(1<<0)|(2<<1),TSD(a0)			; 256 color enable & enable crtc clock divider
	and.b	#~(1<<1),TSD(a0)			; to 24 bit/pixel

	move.b	#CRTC_ExtendedDisplayControls,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#~(1<<7),d0
	or.b	#(1<<5),d0
	move.b	d0,CRTCD(a0)

	moveq	#0,d4		; no border !
	bra	.pixel_end

; 15/16 bit mode adjustments
.pixel_word:
	move.b	#CRTC_ID,(CRTCI,a0)
	move.b	(CRTCD,a0),d0
	lsr.w	#2,d0
	and.w	#%111,d0
	move.b	(HiColorMClks,pc,d0.w),d0
	SetTS	d0,TS_MCLKSelect

	move.b	#TS_ExtendedSequencerMode,TSI(a0)
	or.b	#(1<<0),TSD(a0)				; 256 color enable
	or.b	#(3<<1),TSD(a0)				; enable crtc clock divider to 16 bit/pixel

	move.b	#CRTC_ExtendedDisplayControls,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#~(1<<7),d0
	or.b	#(1<<5),d0
	move.b	d0,CRTCD(a0)

	moveq	#0,d4		; no border !
	bra	.pixel_end

; 8 bit mode adjustments
.pixel_byte:
	move.b	#TS_ExtendedSequencerMode,TSI(a0)
	and.b	#~(3<<1),TSD(a0)			; disable crtc clock divider
	or.b	#(1<<0),TSD(a0)				; 256 color enable
	bra	.pixel_clut

; 4 bit mode adjustments
.pixel_nibble:
	move.b	#TS_ExtendedSequencerMode,TSI(a0)
	and.b	#~(3<<1),TSD(a0)			; disable crtc clock divider
	and.b	#~(1<<0),TSD(a0)			; 256 color disable

; 4 and 8 bit common parts
.pixel_clut:
	move.b	#CRTC_ID,(CRTCI,a0)
	move.b	(CRTCD,a0),d0
	lsr.w	#2,d0
	and.w	#%111,d0
	move.b	(ClutMClks,pc,d0.w),d0
	SetTS	d0,TS_MCLKSelect

	move.b	#CRTC_ExtendedDisplayControls,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#~(1<<7),d0
	or.b	#(1<<5),d0
	tst.w	d4
	beq	.no_border_nibble
	tst.w	gmi_VerBlankSize(a1)
	beq	.no_border_nibble
	tst.w	gmi_HorBlankSize(a1)
	beq	.no_border_nibble
	and.b	#~(1<<5),d0
	or.b	#(1<<7),d0
.no_border_nibble:
	move.b	d0,CRTCD(a0)

; common for all modes
.pixel_end:

	move.w	d3,d0
	lsr.w	#3,d0
	subq.w	#1,d0
	move.b	#CRTC_HorizontalDisplayEnd,CRTCI(a0)
	move.b	d0,CRTCD(a0)

**********************
vertical_display_end:
	moveq	#0,d1
	move.w	gmi_Height(a1),d1
	btst	#GMB_DOUBLESCAN,d2
	beq	.not_doublescan
	add.w	d1,d1
.not_doublescan:
	lsr.w	d6,d1
	subq.w	#1,d1

	move.b	#CRTC_VerticalDisplayEnd,CRTCI(a0)
	move.b	d1,CRTCD(a0)

	move.b	#CRTC_OverflowLow,CRTCI(a0)
	move.b	CRTCD(a0),d0

	and.b	#%10111101,d0
	btst	#8,d1
	beq	.not8
	or.b	#%00000010,d0
.not8:
	btst	#9,d1
	beq	.not9
	or.b	#%01000000,d0
.not9:
	move.b	d0,CRTCD(a0)

*************************************
horizontal_total:
	moveq	#0,d1
	move.w	d5,d1
	move.w	d5,d0
	lsr.w	#3,d1

	subq.w	#5,d1

	move.b	#CRTC_HorizontalTotal,CRTCI(a0)
	move.b	d1,CRTCD(a0)

	lsr.w	#4,d0
	move.b	#CRTC_InterlaceEnd,CRTCI(a0)
	move.b	d0,CRTCD(a0)

*************************************
horizontal_blank:
	moveq	#0,d1
	tst.w	d4
	beq	.no_border_at_left
	move.w	gmi_HorBlankSize(a1),d1
.no_border_at_left:
	add.w	d3,d1
	lsr.w	#3,d1
	subq.w	#1,d1

	move.b	#CRTC_HorizontalBlankStart,CRTCI(a0)
	move.b	d1,CRTCD(a0)

	moveq	#0,d1
	tst.w	d4
	beq	.no_border_at_right
	move.w	gmi_HorBlankSize(a1),d1
	neg.w	d1
.no_border_at_right:
	add.w	d5,d1
	lsr.w	#3,d1
	subq.w	#1,d1

	move.b	#CRTC_MiscellaneousControl,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#%11001111,d0
	btst	#7,d1
	beq	.not7
	bset	#5,d0
.not7:
	btst	#6,d1
	beq	.not6
	bset	#4,d0
.not6:
	move.b	d0,CRTCD(a0)

	move.b	#CRTC_HorizontalSyncEnd,CRTCI(a0)
	move.b	CRTCD(a0),d0
	bclr	#7,d0
	btst	#5,d1
	beq	.not5
	bset	#7,d0
.not5:
	move.b	d0,CRTCD(a0)

	and.b	#%00011111,d1

	move.b	#CRTC_HorizontalBlankEnd,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#%11100000,d0
	or.b	d1,d0
	move.b	d0,CRTCD(a0)

*************************************
horizontal_sync:
	move.w	gmi_HorSyncStart(a1),d1
	add.w	d3,d1
	lsr.w	#3,d1

	move.b	#CRTC_HorizontalSyncStart,CRTCI(a0)
	move.b	d1,CRTCD(a0)

	move.w	gmi_HorSyncSize(a1),d0
	lsr.w	#3,d0
	add.w	d0,d1

	move.b	#CRTC_HorizontalSyncEnd,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#%11100000,d0
	and.b	#%00011111,d1
	or.b	d1,d0
	move.b	d0,CRTCD(a0)

*************************************
horizontal_sync_skew:
	move.b	gmi_HorSyncSkew(a1),d1

	move.b	#CRTC_HorizontalSyncEnd,CRTCI(a0)
	move.b	CRTCD(a0),d0
	lsl.b	#5,d1
	and.b	#%01100000,d1
	and.b	#%10011111,d0
	or.b	d1,d0
	move.b	d0,CRTCD(a0)

*************************************
horizontal_enable_skew:
	move.b	gmi_HorEnableSkew(a1),d1

	move.b	#CRTC_HorizontalBlankEnd,CRTCI(a0)
	move.b	CRTCD(a0),d0
	lsl.b	#5,d1
	and.b	#%01100000,d1
	and.b	#%10011111,d0
	or.b	d1,d0
	move.b	d0,CRTCD(a0)

*************************************
vertical_total:
	moveq	#0,d1
	move.w	gmi_VerTotal(a1),d1
	btst	#GMB_DOUBLESCAN,d2
	beq	.not_doublescan
	add.w	d1,d1
.not_doublescan:
	lsr.w	d6,d1
	subq.w	#2,d1

	move.b	#CRTC_VerticalTotal,CRTCI(a0)
	move.b	d1,CRTCD(a0)

	move.b	#CRTC_OverflowLow,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#%11011110,d0
	btst	#8,d1
	beq	.not8
	bset	#0,d0
.not8:
	btst	#9,d1
	beq	.not9
	bset	#5,d0
.not9:
	move.b	d0,CRTCD(a0)

*************************************
vertical_blank:
	moveq	#0,d1
	move.w	gmi_Height(a1),d1
	btst	#GMB_DOUBLESCAN,d2
	beq	.not_doublescan1
	add.w	d1,d1
.not_doublescan1:
	tst.w	d4
	beq	.no_border_at_bottom
	add.w	gmi_VerBlankSize(a1),d1
.no_border_at_bottom
	lsr.w	d6,d1
	subq.w	#1,d1

	move.b	#CRTC_VerticalBlankStart,CRTCI(a0)
	move.b	d1,CRTCD(a0)

	move.b	#CRTC_OverflowLow,CRTCI(a0)
	move.b	CRTCD(a0),d0
	bclr	#3,d0
	btst	#8,d1
	beq	.not8
	bset	#3,d0
.not8:
	move.b	d0,CRTCD(a0)

	move.b	#CRTC_MaximumRowAddress,CRTCI(a0)
	move.b	CRTCD(a0),d0
	bclr	#5,d0
	btst	#9,d1
	beq	.not9
	bset	#5,d0
.not9:
	move.b	d0,CRTCD(a0)

	move.w	gmi_VerTotal(a1),d1
	btst	#GMB_DOUBLESCAN,d2
	beq	.not_doublescan
	add.w	d1,d1
.not_doublescan:
	tst.w	d4
	beq	.no_border_at_top
	sub.w	gmi_VerBlankSize(a1),d1
.no_border_at_top
	lsr.w	d6,d1
	subq.w	#1,d1

	move.b	#CRTC_MiscellaneousControl,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#%00111111,d0
	btst	#9,d1
	beq	.not_9
	bset	#7,d0
.not_9:
	btst	#8,d1
	beq	.not_8
	bset	#6,d0
.not_8:
	move.b	d0,CRTCD(a0)

	move.b	#CRTC_VerticalBlankEnd,CRTCI(a0)
	move.b	d1,CRTCD(a0)

*************************************
vertical_sync:
	moveq	#0,d1
	move.w	gmi_Height(a1),d1
	btst	#GMB_DOUBLESCAN,d2
	beq	.not_doublescan
	add.w	d1,d1
.not_doublescan:
	add.w	gmi_VerSyncStart(a1),d1
	lsr.w	d6,d1

	move.b	#CRTC_VerticalSyncStart,CRTCI(a0)
	move.b	d1,CRTCD(a0)

	move.b	#CRTC_OverflowLow,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#%01111011,d0
	btst	#8,d1
	beq	.not8
	bset	#2,d0
.not8:
	btst	#9,d1
	beq	.not9
	bset	#7,d0
.not9:
	move.b	d0,CRTCD(a0)

	lsl.w	d6,d1
	add.w	gmi_VerSyncSize(a1),d1
	lsr.w	d6,d1

	move.b	#CRTC_VerticalSyncEnd,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#%00001111,d1
	and.b	#%11110000,d0
	or.b	d1,d0
	move.b	d0,CRTCD(a0)

*************************************
interlace:
	move.b	#CRTC_MiscellaneousControl,CRTCI(a0)
	move.b	CRTCD(a0),d0

	moveq	#0,d1
	bclr	#0,d0
	btst	#GMB_INTERLACE,d2
	beq	.no
	bset	#0,d0
	move.w	gmi_HorSyncStart(a1),d1
	add.w	d3,d1
	lsr.w	#4,d1
.no:
	move.b	d0,CRTCD(a0)

	move.b	#CRTC_InterlaceEnd,CRTCI(a0)
	move.b	d1,CRTCD(a0)

*************************************
doublescan:
	move.b	#CRTC_MaximumRowAddress,CRTCI(a0)
	move.b	CRTCD(a0),d0

	bclr	#7,d0
	btst	#GMB_DOUBLESCAN,d2
	beq	.no
	bset	#7,d0
.no:
	move.b	d0,CRTCD(a0)

*************************************
polarity:

	move.b	MISCOUTPUTR(a0),d0
	and.b	#%00111111,d0

	btst	#GMB_HPOLARITY,d2
	beq	.not_horizontal
	bset	#6,d0
.not_horizontal:
	btst	#GMB_VPOLARITY,d2
	beq	.not_vertical
	bset	#7,d0
.not_vertical:
	move.b	d0,MISCOUTPUTW(a0)

*************************************
	move.l	4,a1
	DISABLE	a1,NOFETCH		; no disturbance while modifying ATC registers

*************************************
	ResetATC

*************************************
bordercolor:

	move.b	ATCI(a0),d0
	and.b	#(1<<5),d0
	or.b	#ATC_OverscanColor,d0
	move.b	d0,ATCI(a0)
	move.b	#0,ATCD(a0)

*************************************
	ENABLE	a1,NOFETCH

*************************************
	POP	d2-d6
	rts


***********************************************************
SetPanning:
* set view origin for overscan displays
***********************************************************
* a0: 	struct BoardInfo
* a1: 	UBYTE *Memory
* d0: 	UWORD Width
* d1: 	WORD XOffset
* d2: 	WORD YOffset
* d7:	RGBFTYPE RGBFormat
***********************************************************
	PUSH	d2-d4

	move.w	d1,(gbi_XOffset,a0)
	move.w	d2,(gbi_YOffset,a0)

	move.l	a1,d4
	sub.l	(gbi_MemoryBase,a0),d4

	cmp.l	#RGBFB_MaxFormats,d7
	bcc	.byte
	
	move.w	(.formats,pc,d7.l*2),d3
	jmp	(.formats,pc,d3.w)

.formats:
	dc.w	.nibble-.formats
	dc.w	.byte-.formats
	dc.w	.tripplebyte-.formats
	dc.w	.tripplebyte-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.byte-.formats
	dc.w	.byte-.formats
	dc.w	.byte-.formats
	dc.w	.byte-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.byte-.formats

.tripplebyte:
	move.w	d0,d3
	add.w	d0,d3
	add.w	d0,d3
	move.w	d3,d0

	lsr.w	#2,d0
	lsr.l	#2,d4
	muls	d0,d2		; linearstartingaddress

	ext.l	d1
	move.l	d1,d0
	add.l	d1,d1
	add.l	d0,d1

	move.l	d1,d0
	asr.l	#2,d0
	add.l	d0,d2

	and.b	#$03,d1		; pixelpanning

	lsr.w	#3,d3		; rowoffset

	bra	.end

.nibble:
	move.w	d0,d3		; rowoffset
	lsr.w	#4,d3

	lsr.w	#3,d0		; linearstartingaddress

	muls	d0,d2
	move.w	d1,d0
	ext.l	d0
	asr.l	#3,d0
	add.l	d0,d2

	and.b	#$07,d1		; pixelpanning

	bra	.end

.byte:
	move.w	d0,d3		; rowoffset
	lsr.w	#3,d3

	lsr.w	#2,d0		; linearstartingaddress
	lsr.l	#2,d4
	muls	d0,d2
	move.w	d1,d0
	ext.l	d0
	asr.l	#2,d0
	add.l	d0,d2

	and.b	#$03,d1		; pixelpanning

	bra	.end

.word:
	move.w	d0,d3		; rowoffset
	lsr.w	#2,d3

	add.w	d0,d0		; linearstartingaddress

	lsr.w	#2,d0
	lsr.l	#2,d4
	muls	d0,d2

	ext.l	d1
	add.l	d1,d1
	move.l	d1,d0
	asr.l	#2,d0
	add.l	d0,d2

	and.b	#$03,d1		; pixelpanning

.end:
	add.l	d4,d2
	move.l	gbi_RegisterBase(a0),a0

**********************
.linear_start_address:
	move.b	#CRTC_LinearStartingAddressLow,CRTCI(a0)
	move.b	d2,CRTCD(a0)

	lsr.w	#8,d2

	move.b	#CRTC_LinearStartingAddressMiddle,CRTCI(a0)
	move.b	d2,CRTCD(a0)
	
	swap	d2

	move.b	#CRTC_ExtendedDisplayControls,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#%11110010,d0
	btst	#0,d2
	beq	.not16
	bset	#0,d0
.not16:
	btst	#1,d2
	beq	.not17
	bset	#2,d0
.not17:
	btst	#2,d2
	beq	.not18
	bset	#3,d0
.not18:
	move.b	d0,CRTCD(a0)

**********************
.row_offset:
	move.b	#CRTC_ExtendedDisplayControls,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#%11101111,d0
	btst	#8,d3
	beq	.not8
	bset	#4,d0
.not8:
	move.b	d0,CRTCD(a0)

	move.b	#CRTC_RowOffset,CRTCI(a0)
	move.b	d3,CRTCD(a0)		; lower eight bits

*************************************
	move.l	4,a1
	DISABLE	a1,NOFETCH		; no disturbance while modifying ATC registers

*************************************
	ResetATC

*************************************
.pixelpanning:

	move.b	ATCI(a0),d0
	and.b	#(1<<5),d0
	or.b	#ATC_HorizontalPixelPanning,d0
	move.b	d0,ATCI(a0)
	move.b	d1,ATCD(a0)

*************************************
	ENABLE	a1,NOFETCH

*************************************
	POP	d2-d4
	rts

***********************************************************
CalculateBytesPerRow:
* calculate length of display line in bytes
***********************************************************
* a0: 	struct BoardInfo
* d0: 	UWORD Width
* d7:	RGBFTYPE RGBFormat
***********************************************************

	cmp.l	#RGBFB_MaxFormats,d7
	bcc	.end
	
	move.w	(.formats,pc,d7.l*2),d1
	jmp	(.formats,pc,d1.w)

.formats:
	dc.w	.nibble-.formats
	dc.w	.byte-.formats
	dc.w	.tripplebyte-.formats
	dc.w	.tripplebyte-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.byte-.formats

.tripplebyte:
	move.w	d0,d1
	add.w	d0,d1
	add.w	d1,d0
	bra	.end

.nibble:
	lsr.w	#3,d0
	bra	.end

.word:
	add.w	d0,d0		; linearstartingaddress
.byte:
.end:
	rts

***********************************************************
CalculateMemory:
***********************************************************
* a0: 	struct BoardInfo
* a1: 	APTR Memory
* d7:	RGBFTYPE RGBFormat
***********************************************************
	move.l	a1,d0
	cmp.l	#RGBFB_PLANAR,d7
	bne	.not_planar
	move.l	(gbi_MemoryBase,a0),d1
	sub.l	d1,d0
	lsr.l	#2,d0
	add.l	d1,d0
.not_planar:
	rts

*************************************
SetSprite:
* a0: 	struct BoardInfo
* d0: 	BOOL activate
* d7:	RGBFTYPE RGBFormat
*************************************
	move.l	gbi_RegisterBase(a0),a0
	move.b	#TS_GraphicsCursorAttributes,TSI(a0)
	move.b	TSD(a0),d1
	and.b	#%11111110,d1
	and.b	#%00000001,d0
	or.b	d0,d1
	move.b	d1,TSD(a0)
	rts

*************************************
SetSpritePosition:
* a0: 	struct BoardInfo
* d7:	RGBFTYPE RGBFormat
*************************************
	PUSH	a2/d2-d6

	move.l	a0,a2

	movem.w	(gbi_MouseX,a2),d4/d5
	sub.w	(gbi_XOffset,a2),d4
	sub.w	(gbi_YOffset,a2),d5

	move.l	gbi_RegisterBase(a2),a0
	move.l	gbi_ModeInfo(a2),d6
	move.l	d6,a1
	beq	.no_mode
	move.b	gmi_Flags(a1),d6

.no_mode:
	btst	#GMB_DOUBLESCAN,d6
	beq	.no_doublescan
	add.w	d5,d5
.no_doublescan:

;	btst	#GMB_INTERLACE,d6
;	beq	.no_interlace
;	asr.w	#1,d5
;.no_interlace:

	tst.w	d4
	bmi	.negative_coordinates

	tst.w	d5
	bpl	.positive_coordinates

.negative_coordinates:
	moveq	#0,d2
	tst.w	d4
	bpl	.x_ok
	sub.w	d4,d2
	moveq	#0,d4
.x_ok:
	moveq	#0,d3
	tst.w	d5
	bpl	.y_ok
	sub.w	d5,d3
	moveq	#0,d5
.y_ok:
	moveq	#0,d0
	move.b	gbi_MouseWidth(a2),d0
	moveq	#0,d1
	move.b	gbi_MouseHeight(a2),d1

	move.l	gbi_Flags(a2),d6
	and.l	#BIF_HIRESSPRITE,d6
	bne	.hires

	move.l	gbi_Flags(a2),d6
	and.l	#BIF_BIGSPRITE,d6
	bne	.zoom
	
	bsr	RenderSpriteNormal
	bra	.done
.hires:
	bsr	RenderSpriteHires
	bra	.done
.zoom:
	bsr	RenderSpriteZoomed
.done:
	move.l	gbi_RegisterBase(a2),a0
	SetTS	#56,TS_GraphicsCursorPattern
	bra	.common

.positive_coordinates:
	move.l	gbi_RegisterBase(a2),a0
	SetTS	#60,TS_GraphicsCursorPattern
.common:

	move.b	#TS_ExtendedSequencerMode,TSI(a0)
	move.b	TSD(a0),d0
	and.b	#(3<<1),d0
	cmp.b	#(3<<1),d0
; let's assume that only the 5428 and 5429 have the hires sprite problem... (tabt)
	bne	.not_16bit
	cmp.b	#CL_GD5428,(gbi_ChipData,a2)
	beq	.16bit
	cmp.b	#CL_GD5429,(gbi_ChipData,a2)
	bne	.not_16bit
.16bit:
	addq.w	#8,d4
.not_16bit:

	andi.w	#$7FF,d4
	andi.w	#$7FF,d5
	lsl.w	#5,d4
	lsl.w	#5,d5
	ori.w	#$10,d4
	ori.w	#$11,d5
	rol.w	#8,d4
	rol.w	#8,d5
	move.w	d4,TSI(a0)	;Cursor X
	move.w	d5,TSI(a0)	;Cursor Y

	POP	a2/d2-d6
	rts

*************************************
SetSpriteImage:
* a0: 	struct BoardInfo
* d7:	RGBFTYPE RGBFormat
*************************************
	PUSH	d2-d4/a2
	move.l	a0,a2

	moveq	#0,d0
	move.b	gbi_MouseWidth(a2),d0
	moveq	#0,d1
	move.b	gbi_MouseHeight(a2),d1
	moveq	#0,d2
	moveq	#0,d3

	move.l	gbi_Flags(a2),d4
	and.l	#BIF_HIRESSPRITE,d4
	bne	.hires

	move.l	gbi_Flags(a2),d4
	and.l	#BIF_BIGSPRITE,d4
	bne	.zoom

	bsr	RenderSpriteNormal
	bra	.done

.hires:
	bsr	RenderSpriteHires
	bra	.done

.zoom:
	bsr	RenderSpriteZoomed

.done:
	tst.w	gbi_MouseX(a2)
	bmi	.render_shifted
	tst.w	gbi_MouseY(a2)
	bpl	.no_render_shifted
.render_shifted:

	move.l	gbi_ModeInfo(a2),a1

	moveq	#0,d2
	move.w	gbi_MouseX(a2),d0
	moveq	#0,d3
	move.w	gbi_MouseY(a2),d1

	move.b	gmi_Flags(a1),d4
	btst	#GMB_DOUBLESCAN,d4
	beq	.no_doublescan
	add.w	d1,d1
.no_doublescan:

;	btst	#GMB_INTERLACE,d4
;	beq	.no_interlace
;	asr.w	#1,d1
;.no_interlace:

	tst.w	d0
	bpl	.x_ok
	sub.w	d0,d2
.x_ok:
	tst.w	d1
	bpl	.y_ok
	sub.w	d1,d3
.y_ok:
	moveq	#0,d0
	move.b	gbi_MouseWidth(a2),d0
	moveq	#0,d1
	move.b	gbi_MouseHeight(a2),d1

	move.l	gbi_Flags(a2),d4
	and.l	#BIF_HIRESSPRITE,d4
	bne	.hires_shifted

	move.l	gbi_Flags(a2),d4
	and.l	#BIF_BIGSPRITE,d4
	bne	.zoom_shifted

	bsr	RenderSpriteNormal
	bra	.done_shifted

.hires_shifted:
	bsr	RenderSpriteHires
	bra	.done_shifted

.zoom_shifted:
	bsr	RenderSpriteZoomed

.done_shifted:
	move.l	gbi_RegisterBase(a2),a0
	SetTS	#56,TS_GraphicsCursorPattern
	bra	.end

.no_render_shifted:
	move.l	gbi_RegisterBase(a2),a0
	SetTS	#60,TS_GraphicsCursorPattern

.end:
	POP	d2-d4/a2
	rts

*************************************
RenderSpriteNormal:
* set sprite image for small sprites (i.e. user did not select "BIGSPRITE=Yes")
* a2: 	struct BoardInfo
* d0.w: Width
* d1.w: Height
* d2.w: OffsetX
* d3.w: OffsetY
*************************************
	PUSH	d2-d4
	move.l	gbi_RegisterBase(a2),a0

	move.b	#GDC_BLTMode,GDCI(a0)
	move.b	#(1<<2),GDCD(a0)			; BLT from system memory

	move.l	gbi_MemorySize(a2),d0
	add.l	#RESERVEDMEMORY-1024,d0
	tst.w	d2
	bne	.negative
	tst.w	d3
	beq	.positive
.negative:
	sub.l	#1024,d0
.positive:
	SetBLTDstStart	d0,a0

	move.w	#(GDC_BLTWidthLow<<8)|255,GDCI(a0)		; 1024 columns
	move.w	#(GDC_BLTWidthHigh<<8)|3,GDCI(a0)

	move.w	#GDC_BLTHeightLow<<8,GDCI(a0)			; 1 row
	move.w	#GDC_BLTHeightHigh<<8,GDCI(a0)

	SetBLTROP	ROP_SRC,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT

	sub.w	d3,d1		; Start line
	move.w	d1,d0

	lsl.w	#2,d3		; YOffset
	move.w	d2,d4

	cmp.w	#64,d1
	ble	.ok
	move.w	#64,d1
.ok:
.plane:
	move.l	gbi_MemoryBase(a2),a0
	move.l	gbi_MouseImage(a2),a1
	add.w	#4,a1
	add.w	d3,a1

	subq.w	#1,d0
	bmi	.plane_done
.plane_loop:
	movem.w	(a1)+,d2/d3
	lsl.w	d4,d3
	lsl.w	d4,d2
	move.w	d3,(a0)
	move.w	#0,(a0)
	move.l	#0,(a0)
	move.w	d2,(a0)
	move.w	#0,(a0)
	move.l	#0,(a0)
	dbra	d0,.plane_loop

.plane_done:
	moveq	#63,d0
	sub.w	d1,d0
	bmi	.plane_clear_done
.plane_clear_loop:
	move.l	#0,(a0)
	move.l	#0,(a0)
	move.l	#0,(a0)
	move.l	#0,(a0)
	dbra	d0,.plane_clear_loop
.plane_clear_done:

	move.l	gbi_RegisterBase(a2),a0
	move.b	#GDC_BLTStartStatus,GDCI(a0)
.wait:	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.wait

	POP	d2-d4
	rts

*************************************
RenderSpriteZoomed:
* set sprite image for big sprites (i.e. user did select "BIGSPRITE=Yes")
* a2: 	struct BoardInfo
* d0.w: Width
* d1.w: Height
* d2.w: OffsetX
* d3.w: OffsetY
*************************************
	PUSH	d2-d7/a3

	lsr.w	#1,d0
	lsr.w	#1,d1

	move.l	gbi_RegisterBase(a2),a0

	move.b	#GDC_BLTMode,GDCI(a0)
	move.b	#(1<<2),GDCD(a0)			; BLT from system memory

	move.l	gbi_MemorySize(a2),d0
	add.l	#RESERVEDMEMORY-1024,d0
	tst.w	d2
	bne	.negative
	tst.w	d3
	beq	.positive
.negative:
	sub.l	#1024,d0
.positive:
	SetBLTDstStart	d0,a0

	move.w	#(GDC_BLTWidthLow<<8)|255,GDCI(a0)		; 1024 columns
	move.w	#(GDC_BLTWidthHigh<<8)|3,GDCI(a0)

	move.w	#GDC_BLTHeightLow<<8,GDCI(a0)			; 1 row
	move.w	#GDC_BLTHeightHigh<<8,GDCI(a0)

	SetBLTROP	ROP_SRC,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT

	move.w	d2,d6		; XOffset

	sub.w	d3,d1		; Start line
	move.w	d1,d0
	add.w	d1,d1
	lsr.w	#1,d3		; carry implies shifted source data access
	bcc	.bit
	bset	#31,d1
	sub.w	#1,d1
.bit:
	move.w	d3,d7		; YOffset
	lsl.w	#2,d7

	cmp.w	#64,d1
	ble	.ok
	move.w	#64,d1
.ok:
.plane:
	move.l	gbi_MemoryBase(a2),a0
	move.l	gbi_MouseImage(a2),a1
	add.w	#4,a1
	add.w	d7,a1

	subq.w	#1,d0
	bmi	.plane_done
.plane_loop:
	move.w	(a1)+,d2
	move.w	d2,d5
	lsr.w	#4,d2
	move.w	d2,d4
	lsr.w	#4,d2
	move.w	d2,d3
	lsr.w	#4,d2
	and.w	#15,d3
	and.w	#15,d4
	and.w	#15,d5
	move.b	.table(pc,d4.w),d4
	swap	d4
	move.b	.table(pc,d2.w),d4
	lsl.l	#8,d4
	move.b	.table(pc,d3.w),d4
	swap	d4
	move.b	.table(pc,d5.w),d4
	lsl.l	d6,d4
	move.l	d4,a3

	move.w	(a1)+,d2
	move.w	d2,d5
	lsr.w	#4,d2
	move.w	d2,d4
	lsr.w	#4,d2
	move.w	d2,d3
	lsr.w	#4,d2
	and.w	#15,d3
	and.w	#15,d4
	and.w	#15,d5
	move.b	.table(pc,d4.w),d4
	swap	d4
	move.b	.table(pc,d2.w),d4
	lsl.l	#8,d4
	move.b	.table(pc,d3.w),d4
	swap	d4
	move.b	.table(pc,d5.w),d4
	lsl.l	d6,d4

	bclr	#31,d1
	bne	.skip
	move.l	d4,(a0)
	move.l	#0,(a0)
	move.l	a3,(a0)
	move.l	#0,(a0)
.skip:
	move.l	d4,(a0)
	move.l	#0,(a0)
	move.l	a3,(a0)
	move.l	#0,(a0)
	dbra	d0,.plane_loop
	bra	.plane_done

.table:
	dc.b	%00000000
	dc.b	%00000011
	dc.b	%00001100
	dc.b	%00001111
	dc.b	%00110000
	dc.b	%00110011
	dc.b	%00111100
	dc.b	%00111111
	dc.b	%11000000
	dc.b	%11000011
	dc.b	%11001100
	dc.b	%11001111
	dc.b	%11110000
	dc.b	%11110011
	dc.b	%11111100
	dc.b	%11111111

.plane_done:
	moveq	#63,d0
	sub.w	d1,d0
	bmi	.plane_clear_done
.plane_clear_loop:
	move.l	#0,(a0)
	move.l	#0,(a0)
	move.l	#0,(a0)
	move.l	#0,(a0)
	dbra	d0,.plane_clear_loop
.plane_clear_done:

	move.l	gbi_RegisterBase(a2),a0
	move.b	#GDC_BLTStartStatus,GDCI(a0)
.wait:	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.wait

	POP	d2-d7/a3
	rts

*************************************
RenderSpriteHires:
* set sprite image for hires sprites
* a2: 	struct BoardInfo
* d0.w: Width
* d1.w: Height
* d2.w: OffsetX
* d3.w: OffsetY
*************************************
	PUSH	d2-d4
	move.l	gbi_RegisterBase(a2),a0

	move.b	#GDC_BLTMode,GDCI(a0)
	move.b	#(1<<2),GDCD(a0)			; BLT from system memory

	move.l	gbi_MemorySize(a2),d0
	add.l	#RESERVEDMEMORY-1024,d0
	tst.w	d2
	bne	.negative
	tst.w	d3
	beq	.positive
.negative:
	sub.l	#1024,d0
.positive:
	SetBLTDstStart	d0,a0

	move.w	#(GDC_BLTWidthLow<<8)|255,GDCI(a0)		; 1024 columns
	move.w	#(GDC_BLTWidthHigh<<8)|3,GDCI(a0)

	move.w	#GDC_BLTHeightLow<<8,GDCI(a0)			; 1 row
	move.w	#GDC_BLTHeightHigh<<8,GDCI(a0)

	SetBLTROP	ROP_SRC,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT

	sub.w	d3,d1		; Start line

	cmp.w	#64,d1
	ble	.ok
	move.w	#64,d1
.ok:

	lsl.w	#3,d3		; YOffset
	move.w	d2,d4

.plane:
	move.l	gbi_MemoryBase(a2),a0
	move.l	gbi_MouseImage(a2),a1
	add.w	#8,a1
	add.w	d3,a1

	move.w	d1,d0
	subq.w	#1,d0
	bmi	.plane_done
.plane_loop:
	movem.l	(a1)+,d2/d3
	lsl.l	d4,d3
	lsl.l	d4,d2
	move.l	d3,(a0)
	move.l	#0,(a0)
	move.l	d2,(a0)
	move.l	#0,(a0)
	dbra	d0,.plane_loop
	bra	.plane_done

.plane_done:
	moveq	#63,d0
	sub.w	d1,d0
	bmi	.plane_clear_done
	moveq	#0,d1
.plane_clear_loop:
	move.l	d1,(a0)
	move.l	d1,(a0)
	move.l	d1,(a0)
	move.l	d1,(a0)
	dbra	d0,.plane_clear_loop
.plane_clear_done:

	move.l	gbi_RegisterBase(a2),a0
	move.b	#GDC_BLTStartStatus,GDCI(a0)
.wait:	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.wait

	POP	d2-d4
	rts

*************************************
SetSpriteColor:
* set sprite colors
* a0: 	struct BoardInfo
* d0.b:	sprite color index (0-2)
* d1.b:	red
* d2.b:	green
* d3.b:	blue
* d7:	RGBFTYPE RGBFormat
*************************************
	tst.b	d0
	beq	.color_0
	cmp.b	#2,d0
	bne	.end
.color_1:
	moveq	#15,d0
.color_0:
	move.l	gbi_RegisterBase(a0),a0

	lsr.b	#2,d1
	lsr.b	#2,d2
	lsr.b	#2,d3
	move.b	d0,PALETTEADDRESS(a0)

	move.b	#TS_GraphicsCursorAttributes,TSI(a0)
	move.b	TSD(a0),d0
	or.b	#2,d0				; reveal extended DAC colors
	move.b	d0,TSD(a0)

	DELAY
	move.b	d1,PALETTEDATA(a0)
	nop
	DELAY
	move.b	d2,PALETTEDATA(a0)
	nop
	DELAY
	move.b	d3,PALETTEDATA(a0)
	nop

	and.b	#~2,d0				; hide extended DAC colors
	move.b	d0,TSD(a0)

.end:
	rts

;-----------------------------------------------

*************************************
SetDAC:
* a0: 	struct BoardInfo
* d7:	RGBFTYPE RGBFormat
*************************************
DACD	equ	$3c6

	move.l	gbi_RegisterBase(a0),a0
	move.b	(.formats,pc,d7.l),d0

	move.b	DACD(a0),d1
	move.b	#0,DACD(a0)
	DELAY
	tst.b	DACD(a0)
	DELAY
	tst.b	DACD(a0)
	DELAY
	tst.b	DACD(a0)
	DELAY
	tst.b	DACD(a0)
	DELAY
	move.b	d0,DACD(a0)	; Hidden DAC
	DELAY
	move.b	d1,DACD(a0)	; Pixel Mask
	rts

.formats:
	dc.b	0,0,$e5,$e5,$e1,$e0
	dc.b	0,0,0,0,0,0
	dc.b	$e1,$e0
	dc.b	0,0

;-----------------------------------------------

*************************************
SetColorArray:
* set color palette
* a0: 	struct BoardInfo
* d0.w:	startindex
* d1.w:	count
*************************************
	PUSH	d2/a6
	move.l	4,a6

	lea	gbi_CLUT(a0),a1
	moveq	#8,d2
	sub.w	gbi_BitsPerCannon(a0),d2
	move.l	gbi_RegisterBase(a0),a0

	DISABLE
	move.b	d0,PALETTEADDRESS(a0)

	add.w	d0,a1
	add.w	d0,a1
	add.w	d0,a1

	bra	.start

.loop:
	move.b	(a1)+,d0
	lsr.b	d2,d0
	move.b	d0,PALETTEDATA(a0)
	nop
	move.b	(a1)+,d0
	lsr.b	d2,d0
	move.b	d0,PALETTEDATA(a0)
	nop
	move.b	(a1)+,d0
	lsr.b	d2,d0
	move.b	d0,PALETTEDATA(a0)
	nop
.start:
	dbra	d1,.loop

	ENABLE
	POP	d2/a6
	rts

*************************************
SetDPMSLevel:
* sets degrees of power management
* a0: 	struct BoardInfo
* d0.l:	DPMS level
*************************************

	cmp.l	#DPMS_OFF,d0
	bgt	.end

*************************************

	PUSH	d2
	move.l	d0,d2

	move.l	gbi_ModeInfo(a0),a1
	move.l	gbi_RegisterBase(a0),a0

	move.b	MISCOUTPUTR(a0),d1
	and.b	#%00111111,d1

	cmp.l	#DPMS_ON,d2
	bne	.special_polarity

	btst	#GMB_HPOLARITY,gmi_Flags(a1)
	beq	.not_horizontal
	bset	#6,d1
.not_horizontal:
	btst	#GMB_VPOLARITY,gmi_Flags(a1)
	beq	.not_vertical
	bset	#7,d1
.not_vertical:
.special_polarity:
	move.b	d1,MISCOUTPUTW(a0)

	move.b	#CRTC_CRTCMode,CRTCI(a0)
	move.b	CRTCD(a0),d1
	or.b	#%10000000,d1

	cmp.l	#DPMS_OFF,d2		; Lowest level of power consumption
	bne	.not_off

	and.b	#%01111111,d1		; disable timing
.not_off:
	move.b	d1,CRTCD(a0)

.vsync:
	moveq	#-1,d0
	moveq	#0,d1

	cmp.l	#DPMS_SUSPEND,d2	; Significant reduction of power consumption
	beq	.vsync_static

.vsync_normal:
	move.w	gmi_Height(a1),d0
	btst	#GMB_DOUBLESCAN,gmi_Flags(a1)
	beq	.not_doublescan
	add.w	d0,d0
.not_doublescan:
	add.w	gmi_VerSyncStart(a1),d0
	move.w	d0,d1
	add.w	gmi_VerSyncSize(a1),d1

	btst	#GMB_INTERLACE,gmi_Flags(a1)
	bne	.interlace
	btst	#GMB_DOUBLEVERTICAL,gmi_Flags(a1)
	beq	.not_doublevertical
.interlace:
	lsr.w	#1,d0
	lsr.w	#1,d1
.not_doublevertical:

.vsync_static:
	move.b	#CRTC_VerticalSyncStart,CRTCI(a0)
	move.b	d0,CRTCD(a0)

	move.b	#CRTC_OverflowLow,CRTCI(a0)
	move.b	CRTCD(a0),d2
	and.b	#%01111011,d2
	btst	#8,d0
	beq	.not8
	bset	#2,d2
.not8:
	btst	#9,d0
	beq	.not9
	bset	#7,d2
.not9:
	move.b	d2,CRTCD(a0)

	move.b	#CRTC_VerticalSyncEnd,CRTCI(a0)
	move.b	CRTCD(a0),d0
	and.b	#%00001111,d1
	and.b	#%11110000,d0
	or.b	d1,d0
	move.b	d0,CRTCD(a0)

*************************************

	POP	d2
.end:
	rts


*************************************
WaitBlitter030:
* wait until blitter is ready (030 WriteAllocate Bug)
* a0: 	struct BoardInfo
*************************************
	PUSH	a2/a6
	move.l	(gbi_ExecBase,a0),a6

	move.l	(gbi_RegisterBase,a0),a0
	move.b	#GDC_BLTStartStatus,GDCI(a0)
	nop
	lea	GDCD(a0),a2
.wait:
	move.l	a2,a0
	moveq	#1,d0
	move.l	#CACRF_ClearD,d1
	CALL	CacheClearE
	btst	#3,(a2)
	bne	.wait

	POP	a2/a6
	rts


*************************************
WaitBlitter:
* wait until blitter is ready (020/040/060)
* a0: 	struct BoardInfo
*************************************

	move.l	gbi_RegisterBase(a0),a0
	move.b	#GDC_BLTStartStatus,GDCI(a0)
	nop
.wait:
	btst	#3,GDCD(a0)		; BLT Progress Status
	bne	.wait
	rts

***********************************************************
LargeBlit:
***********************************************************
* a0: 	RegisterBase
* a1:	Destination Address (board relative)
* d2: 	WORD width
* d3: 	WORD height
***********************************************************
	PUSH	d0-d3

	move.w	#MAX_BLTWIDTH,d1
	cmp.w	d1,d2
	ble	.width_ok

	move.w	#MAX_BLTWIDTH,d0
	SetBLTWidth	d0,a0

.big_width_loop:
	sub.w	d1,d2

	move.l	a1,d0
	SetBLTDstStart	d0,a0

	add.w	d1,a1			; add max width to dst memory for next blit

	move.w	d3,d0
	SetBLTHeight	d0,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
.wait1:
	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.wait1

	cmp.w	d1,d2
	bgt	.big_width_loop

	move.l	a1,d0
	SetBLTDstStart	d0,a0

	SetBLTWidth	d2,a0
	SetBLTHeight	d3,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	bra	.end

.width_ok:
	move.l	a1,d0
	SetBLTDstStart	d0,a0

	SetBLTWidth	d2,a0

	move.w	#MAX_BLTHEIGHT,d1
.check_big_height:
	cmp.w	d1,d3
	ble	.height_ok
	sub.w	d1,d3

	move.w	#MAX_BLTHEIGHT,d0
	SetBLTHeight	d0,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
.wait2:	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.wait2
	bra	.check_big_height

.height_ok:
	SetBLTHeight	d3,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
.end:
	POP	d0-d3
	rts


***********************************************************
BlitRect:
***********************************************************
* a0: 	struct BoardInfo
* a1:	struct RenderInfo
* d0: 	WORD SrcX
* d1: 	WORD SrcY
* d2: 	WORD DstX
* d3: 	WORD DstY
* d4: 	WORD SizeX
* d5: 	WORD SizeY
* d6:	UBYTE Mask
* d7:	ULONG RGBFormat
***********************************************************
	PUSH	a2-a3/d2-d7

	cmp.l	#RGBFB_MaxFormats,d7
	bcc	.end

	move.l	a0,a3

	move.w	(.formats,pc,d7.l*2),d7
	jmp	(.formats,pc,d7.w)

.formats:
	dc.w	.end-.formats
	dc.w	.byte-.formats
	dc.w	.triplebyte-.formats
	dc.w	.triplebyte-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.byte-.formats

.triplebyte:
	move.w	d0,d7
	add.w	d0,d0
	add.w	d7,d0
	move.w	d2,d7
	add.w	d2,d2
	add.w	d7,d2
	move.w	d4,d7
	add.w	d4,d4
	add.w	d7,d4
	moveq	#-1,d6
	bra	.all_formats

.word:
	add.w	d0,d0
	add.w	d2,d2
	add.w	d4,d4
	moveq	#-1,d6
	bra	.all_formats

.byte:
	move.b	d6,d7
	swap	d6
	move.b	d7,d6
	lsl.l	#8,d6
	move.b	d7,d6
	swap	d6
	move.b	d7,d6
;	bra	.all_formats

.all_formats:

	move.l	d6,d7
	move.w	(gri_BytesPerRow,a1),d6
	move.l	(gri_Memory,a1),a1
	sub.l	(gbi_MemoryBase,a3),a1	; vga relative start address
	move.l	a1,a2

	cmp.w	d1,d3
	beq	.copybitmap_horizontal
	blt	.copybitmap_down
.copybitmap_up:
	cmp.w	#MAX_BLTWIDTH,d4
	ble	.up_width_ok
	cmp.w	d0,d2
	bgt	.copybitmap_left

	add.w	d5,d1
	subq.w	#1,d1

	add.w	#MAX_BLTWIDTH,d0
	subq.w	#1,d0

	add.w	d0,a1
	muls	d6,d1
	add.l	d1,a1			; vga memory start address

	move.w	d3,d0
	add.w	d5,d0
	subq.w	#1,d0
	muls	d6,d0
	add.l	d0,a2
	add.w	d2,a2			; vga memory start address
	add.w	#MAX_BLTWIDTH,a2
	sub.w	#1,a2

	moveq	#1,d0
	move.w	d4,d2
	sub.w	#MAX_BLTWIDTH,d2
	bra	.copycases_done

.copybitmap_horizontal:
	cmp.w	d0,d2
	blt	.copybitmap_right

.up_width_ok:
.copybitmap_left:
	add.w	d5,d1
	subq.w	#1,d1

	add.w	d4,d0
	subq.w	#1,d0

	add.w	d0,a1
	muls	d6,d1
	add.l	d1,a1			; vga memory start address

	move.w	d3,d0
	add.w	d5,d0
	subq.w	#1,d0
	muls	d6,d0
	add.l	d0,a2
	add.w	d2,a2			; vga memory start address
	add.w	d4,a2
	sub.w	#1,a2

	moveq	#1,d0
	move.w	#-MAX_BLTWIDTH,d2
	bra	.copycases_done

.copybitmap_down:
	cmp.w	#MAX_BLTWIDTH,d4
	ble	.down_width_ok
	cmp.w	d0,d2
	blt	.copybitmap_right

	add.w	d0,a1
	add.w	d4,a1
	sub.w	#MAX_BLTWIDTH,a1
	muls	d6,d1
	add.l	d1,a1			; vga memory start address

	move.w	d3,d0
	muls	d6,d0
	add.l	d0,a2
	add.w	d2,a2			; vga memory start address
	add.w	d4,a2
	sub.w	#MAX_BLTWIDTH,a2

	moveq	#0,d0
	move.w	#MAX_BLTWIDTH,d2
	sub.w	d4,d2
	bra	.copycases_done


.down_width_ok:
.copybitmap_right:
	add.w	d0,a1
	muls	d6,d1
	add.l	d1,a1			; vga memory start address

	move.w	d3,d0
	muls	d6,d0
	add.l	d0,a2
	add.w	d2,a2			; vga memory start address

	moveq	#0,d0
	move.w	#MAX_BLTWIDTH,d2
.copycases_done:

	move.l	gbi_Flags(a3),d1
	and.l	#BIF_IGNOREMASK,d1
	bne	.ignore_mask

	cmp.l	#$ffffffff,d7
	bne	.maskblit

.ignore_mask:
	move.l	(gbi_RegisterBase,a3),a0

	move.b	#GDC_BLTMode,GDCI(a0)
	move.b	d0,GDCD(a0)

	move.w	d6,d0
	SetBLTSrcPitch	d0,a0

	move.w	d6,d0
	SetBLTDstPitch	d0,a0

	SetBLTROP	ROP_SRC,a0

	move.l	a1,d0
	SetBLTSrcStart	d0,a0
	move.l	a2,d0
	SetBLTDstStart	d0,a0

	move.w	#MAX_BLTWIDTH,d1
	cmp.w	d1,d4
	ble	.width_ok

.big_width_loop:
	sub.w	d1,d4

	move.w	#MAX_BLTWIDTH,d0
	SetBLTWidth	d0,a0
	move.w	d5,d0
	SetBLTHeight	d0,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	nop
.wait1:	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.wait1

	add.w	d2,a1			; add max width to dst memory for next blit
	move.l	a1,d0
	SetBLTSrcStart	d0,a0
	add.w	d2,a2			; add max width to dst memory for next blit
	move.l	a2,d0
	SetBLTDstStart	d0,a0

	SetBLTWidth	d4,a0
	SetBLTHeight	d5,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	nop
	bra	.end

.width_ok:
	SetBLTWidth	d4,a0

	move.w	#MAX_BLTHEIGHT,d1
.check_big_height:
	cmp.w	d1,d5
	ble	.height_ok
	sub.w	d1,d5

	move.w	#MAX_BLTHEIGHT,d0
	SetBLTHeight	d0,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	nop
.wait2:	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.wait2
	bra	.check_big_height

.height_ok:
	SetBLTHeight	d5,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	nop
	bra	.end

;-------------------------------------------------------
.maskblit:
;-------------------------------------------------------

	move.l	gbi_MemorySize(a3),a0
	add.l	gbi_MemoryBase(a3),a0
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+
	move.l	d7,(a0)+

	move.l	(gbi_RegisterBase,a3),a0

	PUSH	a4-a6
	move.w	#MAX_BLTWIDTH,d1
	cmp.w	d1,d4
	ble	.maskblit_width_ok

	PUSH	a0-a3/d0-d7
	move.w	#MAX_BLTWIDTH,d4
	bsr	.maskblit_column
	POP	a0-a3/d0-d7

	sub.w	#MAX_BLTWIDTH,d4
	add.w	d2,a1			; add max width to dst memory for next blit
	add.w	d2,a2			; add max width to dst memory for next blit
.maskblit_width_ok:
	bsr	.maskblit_column
	POP	a4-a6

.end:
*************************************
	POP	a2-a3/d2-d7
	rts

*************************************

.maskblit_column:
	move.l	(gbi_MemorySize,a3),a3
	move.l	a3,d7
	and.l	#$ffffff,d7
	or.l	#(GDC_BLTSrcStartHigh<<24),d7
	move.l	d7,a3

	add.l	#64,d7
	move.w	#GDC_BLTDstStartLow<<8,d1
	move.b	d7,d1
	move.w	d1,a4
	lsr.w	#8,d7
	move.w	#GDC_BLTDstStartMiddle<<8,d1
	move.b	d7,d1
	move.w	d1,a5
	swap	d7
	move.w	#GDC_BLTDstStartHigh<<8,d1
	move.b	d7,d1
	move.w	d1,a6

	move.l	#RESERVEDMEMORY-(2*1024+64),d1
	divu	d4,d1			; max lines in one run
	and.l	#$ffff,d5
	divu	d1,d5			; number of full runs
	move.l	d5,d3
	swap	d3			; lines in first run

	subq.w	#1,d4
	moveq	#0,d2
	move.w	d6,d2			; modulo (unmodified) for blitter

	move.w	d6,d7
	mulu	d3,d6
	mulu	d1,d7

	tst.l	d0
	beq	.incrementing
	sub.w	d4,a1
	sub.w	d4,a2
	sub.l	d2,d6
	bset	#31,d2
.incrementing:

	addq.w	#1,d4
	move.w	d4,d0
	SetBLTWidth	d0,a0

	and.w	#%0000111111111111,d2
	and.w	#%0000111111111111,d4

	move.l	a1,d0
	and.l	#$ffffff,d0
	or.l	#(GDC_BLTSrcStartHigh<<24),d0
	move.l	d0,a1

	subq.w	#1,d1
	and.l	#4*256-1,d1
	lsl.l	#8,d1
	lsr.w	#8,d1
	or.w	#(GDC_BLTHeightLow<<8),d1
	or.l	#(GDC_BLTHeightHigh<<24),d1

	subq.w	#1,d3			; anything to do in first turn?
	bmi	.loop_end

	and.l	#4*256-1,d3
	lsl.l	#8,d3
	lsr.w	#8,d3
	or.w	#(GDC_BLTHeightLow<<8),d3
	or.l	#(GDC_BLTHeightHigh<<24),d3

.y_loop:
	tst.l	d2
	bpl	.pos
	sub.l	d6,a1
	sub.l	d6,a2
.pos:

	; copy dst to temp

	move.w	d3,GDCI(a0)
	swap	d3
	move.w	d3,GDCI(a0)
	swap	d3

	move.w	d2,d0
	SetBLTSrcPitch	d0,a0

	move.w	d4,d0
	SetBLTDstPitch	d0,a0

	move.l	a2,d0
	SetBLTSrcStart	d0,a0

	move.w	a4,GDCI(a0)
	move.w	a5,GDCI(a0)
	move.w	a6,GDCI(a0)

	move.w	#GDC_BLTMode<<8,GDCI(a0)
	SetBLTROP	ROP_SRC,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
.wait_copy:
	move.b	GDCD(a0),d0
	btst	#3,d0					; blitter progress status
	bne	.wait_copy

	; neor src to temp

	move.w	d3,GDCI(a0)
	swap	d3
	move.w	d3,GDCI(a0)
	swap	d3

	move.l	a1,d0
	move.b	#GDC_BLTSrcStartLow,GDCI(a0)
	move.b	d0,GDCD(a0)
	lsr.w	#8,d0
	move.b	#GDC_BLTSrcStartMiddle,GDCI(a0)
	move.b	d0,GDCD(a0)
	swap	d0
	move.w	d0,GDCI(a0)				; carefull !!! (high byte is index)

	move.w	a4,GDCI(a0)
	move.w	a5,GDCI(a0)
	move.w	a6,GDCI(a0)

	SetBLTROP	ROP_NEOR,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
.wait_neor1:
	move.b	GDCD(a0),d0
	btst	#3,d0					; blitter progress status
	bne	.wait_neor1

	; or mask to temp

	move.w	d3,GDCI(a0)
	swap	d3
	move.w	d3,GDCI(a0)
	swap	d3

	move.l	a3,d0
	move.b	#GDC_BLTSrcStartLow,GDCI(a0)
	move.b	d0,GDCD(a0)
	lsr.w	#8,d0
	move.b	#GDC_BLTSrcStartMiddle,GDCI(a0)
	move.b	d0,GDCD(a0)
	swap	d0
	move.w	d0,GDCI(a0)				; carefull !!! (high byte is index)

	move.w	a4,GDCI(a0)
	move.w	a5,GDCI(a0)
	move.w	a6,GDCI(a0)

	move.w	#(GDC_BLTMode<<8)|(1<<6),GDCI(a0)	; 8x8 Pattern Copy
	SetBLTROP	ROP_OR,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
.wait_or:
	move.b	GDCD(a0),d0
	btst	#3,d0					; blitter progress status
	bne	.wait_or

	; neor src to modified tmp

	move.w	d3,GDCI(a0)
	swap	d3
	move.w	d3,GDCI(a0)
	swap	d3

	move.l	a1,d0
	move.b	#GDC_BLTSrcStartLow,GDCI(a0)
	move.b	d0,GDCD(a0)
	lsr.w	#8,d0
	move.b	#GDC_BLTSrcStartMiddle,GDCI(a0)
	move.b	d0,GDCD(a0)
	swap	d0
	move.w	d0,GDCI(a0)				; carefull !!! (high byte is index)

	move.w	a4,GDCI(a0)
	move.w	a5,GDCI(a0)
	move.w	a6,GDCI(a0)

	SetBLTROP	ROP_NEOR,a0
	move.w	#GDC_BLTMode<<8,GDCI(a0)

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
.wait_neor2:
	move.b	GDCD(a0),d0
	btst	#3,d0					; blitter progress status
	bne	.wait_neor2

	; copy new dst back

	move.w	d3,GDCI(a0)
	swap	d3
	move.w	d3,GDCI(a0)
	swap	d3

	move.w	d4,d0
	SetBLTSrcPitch	d0,a0

	move.w	d2,d0
	SetBLTDstPitch	d0,a0

	move.l	a3,d0
	add.l	#64,d0
	move.b	#GDC_BLTSrcStartLow,GDCI(a0)
	move.b	d0,GDCD(a0)
	lsr.w	#8,d0
	move.b	#GDC_BLTSrcStartMiddle,GDCI(a0)
	move.b	d0,GDCD(a0)
	swap	d0
	move.w	d0,GDCI(a0)				; carefull !!! (high byte is index)

	move.l	a2,d0
	SetBLTDstStart	d0,a0

	SetBLTROP	ROP_SRC,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
.wait_copyback:
	move.b	GDCD(a0),d0
	btst	#3,d0					; blitter progress status
	bne	.wait_copyback

	tst.l	d2
	bmi	.neg
	add.l	d6,a1
	add.l	d6,a2
.neg:
.loop_end:
	move.l	d1,d3
	move.l	d7,d6

	dbra	d5,.y_loop
	rts

*************************************


***********************************************************

ROP_table:
	dc.b	ROP_FALSE 	; (-> FillRect)
	dc.b	ROP_NOR
	dc.b	ROP_ONLYDST	; (~s /\ d)
	dc.b	ROP_NOTSRC
	dc.b	ROP_ONLYSRC	; (s /\ ~d)
	dc.b	ROP_NOTDST	; (-> InvertRect)
	dc.b	ROP_EOR
	dc.b	ROP_NAND
	dc.b	ROP_AND
	dc.b	ROP_NEOR
	dc.b	ROP_DST		; (nop)
	dc.b	ROP_NOTONLYSRC	; (~s \/ d)
	dc.b	ROP_SRC
	dc.b	ROP_NOTONLYDST	; (s \/ ~d)
	dc.b	ROP_OR
	dc.b	ROP_TRUE	; (-> FillRect)

***********************************************************
BlitRectNoMaskComplete:
***********************************************************
* a0: 	struct BoardInfo
* a1:	struct RenderInfo *sri
* a2:	struct RenderInfo *dri
* d0: 	WORD SrcX
* d1: 	WORD SrcY
* d2: 	WORD DstX
* d3: 	WORD DstY
* d4: 	WORD SizeX
* d5: 	WORD SizeY
* d6:	UBYTE Opcode
* d7:	ULONG RGBFormat
***********************************************************

	PUSH	a2-a3/d2-d7
*************************************

	cmp.l	#RGBFB_MaxFormats,d7
	bcc	.end

	move.l	a0,a3
	swap	d6			; opcode -> hiword

	move.w	(.formats,pc,d7.l*2),d7
	jmp	(.formats,pc,d7.w)

.formats:
	dc.w	.end-.formats
	dc.w	.byte-.formats
	dc.w	.triplebyte-.formats
	dc.w	.triplebyte-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.byte-.formats

.triplebyte:
	move.w	d0,d7
	add.w	d0,d0
	add.w	d7,d0
	move.w	d2,d7
	add.w	d2,d2
	add.w	d7,d2
	move.w	d4,d7
	add.w	d4,d4
	add.w	d7,d4
	bra	.all_formats

.word:
	add.w	d0,d0
	add.w	d2,d2
	add.w	d4,d4
	bra	.all_formats

.byte:
;	bra	.all_formats

.all_formats:

; src
	move.w	(gri_BytesPerRow,a1),d6
	move.l	(gri_Memory,a1),a1
	sub.l	(gbi_MemoryBase,a3),a1	; vga relative start address
; dest
	move.w	(gri_BytesPerRow,a2),d7
	move.l	(gri_Memory,a2),a2
	sub.l	(gbi_MemoryBase,a3),a2	; vga relative start address

	cmp.l	a1,a2
	bne	.copybitmap_down	; simple case, different locations, e.g. ClipRects

	cmp.w	d1,d3
	beq	.copybitmap_horizontal
	blt	.copybitmap_down
.copybitmap_up:
	cmp.w	#MAX_BLTWIDTH,d4
	ble	.up_width_ok
	cmp.w	d0,d2
	bgt	.copybitmap_left

	add.w	d5,d1
	subq.w	#1,d1

	add.w	#MAX_BLTWIDTH,d0
	subq.w	#1,d0

	add.w	d0,a1
	muls	d6,d1
	add.l	d1,a1			; vga memory start address

	move.w	d3,d0
	add.w	d5,d0
	subq.w	#1,d0
	muls	d7,d0
	add.l	d0,a2
	add.w	d2,a2			; vga memory start address
	add.w	#MAX_BLTWIDTH,a2
	sub.w	#1,a2

	moveq	#1,d0
	move.w	d4,d2
	sub.w	#MAX_BLTWIDTH,d2
	bra	.copycases_done

.copybitmap_horizontal:
	cmp.w	d0,d2
	blt	.copybitmap_right

.up_width_ok:
.copybitmap_left:
	add.w	d5,d1
	subq.w	#1,d1

	add.w	d4,d0
	subq.w	#1,d0

	add.w	d0,a1
	muls	d6,d1
	add.l	d1,a1			; vga memory start address

	move.w	d3,d0
	add.w	d5,d0
	subq.w	#1,d0
	muls	d7,d0
	add.l	d0,a2
	add.w	d2,a2			; vga memory start address
	add.w	d4,a2
	sub.w	#1,a2

	moveq	#1,d0
	move.w	#-MAX_BLTWIDTH,d2
	bra	.copycases_done

.copybitmap_down:
	cmp.w	#MAX_BLTWIDTH,d4
	ble	.down_width_ok
	cmp.w	d0,d2
	blt	.copybitmap_right

	add.w	d0,a1
	add.w	d4,a1
	sub.w	#MAX_BLTWIDTH,a1
	muls	d6,d1
	add.l	d1,a1			; vga memory start address

	move.w	d3,d0
	muls	d7,d0
	add.l	d0,a2
	add.w	d2,a2			; vga memory start address
	add.w	d4,a2
	sub.w	#MAX_BLTWIDTH,a2

	moveq	#0,d0
	move.w	#MAX_BLTWIDTH,d2
	sub.w	d4,d2
	bra	.copycases_done


.down_width_ok:
.copybitmap_right:
	add.w	d0,a1
	muls	d6,d1
	add.l	d1,a1			; vga memory start address

	move.w	d3,d0
	muls	d7,d0
	add.l	d0,a2
	add.w	d2,a2			; vga memory start address

	moveq	#0,d0
	move.w	#MAX_BLTWIDTH,d2
.copycases_done:

	move.l	(gbi_RegisterBase,a3),a0

	move.b	#GDC_BLTMode,GDCI(a0)
	move.b	d0,GDCD(a0)

	move.w	d6,d0
	SetBLTSrcPitch	d0,a0

	move.w	d7,d0
	SetBLTDstPitch	d0,a0

	moveq	#0,d0
	swap	d6
	move.b	d6,d0
	move.b	(ROP_table,pc,d0.w),d0
	SetGDC	d0,GDC_BLTRasterOperation

	move.l	a1,d0
	SetBLTSrcStart	d0,a0
	move.l	a2,d0
	SetBLTDstStart	d0,a0

	move.w	#MAX_BLTWIDTH,d1
	cmp.w	d1,d4
	ble	.width_ok

.big_width_loop:
	sub.w	d1,d4

	move.w	#MAX_BLTWIDTH,d0
	SetBLTWidth	d0,a0
	move.w	d5,d0
	SetBLTHeight	d0,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	nop
.wait1:	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.wait1

	add.w	d2,a1			; add max width to dst memory for next blit
	move.l	a1,d0
	SetBLTSrcStart	d0,a0
	add.w	d2,a2			; add max width to dst memory for next blit
	move.l	a2,d0
	SetBLTDstStart	d0,a0

	SetBLTWidth	d4,a0
	SetBLTHeight	d5,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	nop
	bra	.end

.width_ok:
	SetBLTWidth	d4,a0

	move.w	#MAX_BLTHEIGHT,d1
.check_big_height:
	cmp.w	d1,d5
	ble	.height_ok
	sub.w	d1,d5

	move.w	#MAX_BLTHEIGHT,d0
	SetBLTHeight	d0,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	nop
.wait2:	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.wait2
	bra	.check_big_height

.height_ok:
	SetBLTHeight	d5,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	nop

.end:
*************************************
	POP	a2-a3/d2-d7
	rts


***********************************************************
FillRect:
***********************************************************
* a0: 	struct BoardInfo
* a1:	struct RenderInfo
* d0: 	WORD X
* d1: 	WORD Y
* d2: 	WORD Width
* d3: 	WORD Height
* d4:	ULONG Pen
* d5:	UBYTE Mask
* d7:	ULONG RGBFormat
***********************************************************

	PUSH	a2/d2-d7

	cmp.l	#RGBFB_MaxFormats,d7
	bcc	.end

	move.l	a0,a2
	move.l	(gbi_RegisterBase,a0),a0

	move.w	(gri_BytesPerRow,a1),d6
	SetBLTDstPitch	d6,a0
	move.w	(gri_BytesPerRow,a1),d6
	move.l	(gri_Memory,a1),a1

	move.w	(.formats,pc,d7.l*2),d7
	jmp	(.formats,pc,d7.w)

.formats:
	dc.w	.end-.formats
	dc.w	.byte-.formats
	dc.w	.triplebyte-.formats
	dc.w	.triplebyte-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.word-.formats
	dc.w	.byte-.formats

*************************************
.triplebyte:
	muls	d6,d1
	add.l	d1,a1
	add.w	d0,a1
	add.w	d0,a1
	add.w	d0,a1
	move.l	a1,d1				; vga operation start address
	sub.l	(gbi_MemoryBase,a2),d1

	move.l	(gbi_MemoryBase,a2),a1
	add.l	(gbi_MemorySize,a2),a1

	move.w	d2,d0
	add.w	d2,d2
	add.w	d0,d2

	move.w	#GDC_BLTSrcPitchLow<<8,GDCI(a0)
	move.w	#GDC_BLTSrcPitchHigh<<8,GDCI(a0)

	;---------------------------------------
	; fill one line to be repeated
	;---------------------------------------

	move.l	d4,d0	; -bgr
	rol.w	#8,d4	; -brg
	swap	d4	; rg-b
	lsl.w	#8,d4	; rgb-
	move.b	d0,d4	; rgbr
	move.l	d4,d5	; rgbr
	lsl.l	#8,d5	; gbr-
	lsr.l	#8,d0	; --bg
	move.b	d0,d5	; gbrg
	move.l	d5,d7	; gbrg
	lsl.l	#8,d7	; brg-
	lsr.l	#8,d0	; ---b
	move.b	d0,d7	; brgb

	move.w	d2,d0
	subq.w	#1,d0
	lsr.w	#2,d0

.line_loop:
	move.l	d4,(a1)+
	subq.w	#1,d0
	bmi	.line_done
	move.l	d5,(a1)+
	subq.w	#1,d0
	bmi	.line_done
	move.l	d7,(a1)+
	subq.w	#1,d0
	bpl	.line_loop
.line_done:

	move.w	#GDC_BLTMode<<8,GDCI(a0)
	SetBLTROP	ROP_SRC,a0

	move.l	(gbi_MemorySize,a2),a2
	move.l	d1,a1

	move.w	#MAX_BLTWIDTH,d1
	cmp.w	d1,d2
	ble	.width_ok

	move.w	#MAX_BLTWIDTH,d0
	SetBLTWidth	d0,a0

.big_width_loop:
	sub.w	d1,d2

	move.l	a2,d0
	SetBLTSrcStart	d0,a0
	move.l	a1,d0
	SetBLTDstStart	d0,a0

	add.w	d1,a1			; add max width to dst memory for next blit
	add.w	d1,a2			; add max width to src memory for next blit

	move.w	d3,d0
	SetBLTHeight	d0,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
.wait1:	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.wait1

	cmp.w	d1,d2
	bgt	.big_width_loop

	move.l	a2,d0
	SetBLTSrcStart	d0,a0
	move.l	a1,d0
	SetBLTDstStart	d0,a0

	SetBLTWidth	d2,a0
	SetBLTHeight	d3,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	bra	.end

.width_ok:
	move.l	a2,d0
	SetBLTSrcStart	d0,a0
	move.l	a1,d0
	SetBLTDstStart	d0,a0

	SetBLTWidth	d2,a0

	move.w	#MAX_BLTHEIGHT,d1
.check_big_height:
	cmp.w	d1,d3
	ble	.height_ok
	sub.w	d1,d3

	move.w	#MAX_BLTHEIGHT,d0
	SetBLTHeight	d0,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
.wait2:	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.wait2
	bra	.check_big_height

.height_ok:
	SetBLTHeight	d3,a0

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	bra	.end

*************************************
.word:
	add.w	d2,d2

	muls	d6,d1
	add.l	d1,a1
	add.w	d0,a1
	add.w	d0,a1
	move.l	a1,d1				; vga operation start address
	sub.l	(gbi_MemoryBase,a2),d1

	move.w	#(GDC_BLTMode<<8)|(1<<6),GDCI(a0)	; 8x8 Pattern Copy

	bra	.start_word

*************************************
.byte:
	move.b	d4,d7
	swap	d4
	move.b	d7,d4
	lsl.l	#8,d4
	move.b	d7,d4
	swap	d4
	move.b	d7,d4
	move.b	d5,d7
	swap	d5
	move.b	d7,d5
	lsl.l	#8,d5
	move.b	d7,d5
	swap	d5
	move.b	d7,d5

	muls	d6,d1
	add.l	d1,a1
	add.w	d0,a1
	move.l	a1,d1				; vga operation start address
	sub.l	(gbi_MemoryBase,a2),d1

	move.w	#(GDC_BLTMode<<8)|(1<<6),GDCI(a0)	; 8x8 Pattern Copy

	cmp.l	#$ffffffff,d5
	beq	.full_mask

	move.l	(gbi_MemorySize,a2),d0
	SetBLTSrcStart	d0,a0

	and.l	d5,d4

	move.l	gbi_MemorySize(a2),a1
	add.l	gbi_MemoryBase(a2),a1
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+
	move.l	d5,(a1)+

	SetBLTROP	ROP_ONLYDST,a0

	move.l	d1,a1
	bsr	LargeBlit

	move.b	#GDC_BLTStartStatus,GDCI(a0)
.wait:	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.wait

	SetBLTROP	ROP_OR,a0
	bra	.used_mask

.full_mask:
*************************************
.start_word:

	SetBLTROP	ROP_SRC,a0

.used_mask:
	move.l	gbi_MemorySize(a2),a1
	add.l	gbi_MemoryBase(a2),a1
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+

	move.l	(gbi_MemorySize,a2),d0
	SetBLTSrcStart	d0,a0

	move.l	d1,a1
	bsr	LargeBlit

.end:
	POP	a2/d2-d7
	rts


***********************************************************
InvertRect:
***********************************************************
* a0: 	struct BoardInfo
* a1:	struct RenderInfo
* d0: 	WORD X
* d1: 	WORD Y
* d2: 	WORD Width
* d3: 	WORD Height
* d4:	UBYTE Mask
* d7:	ULONG RGBFormat
***********************************************************

	PUSH	a2/d2/d4-d5

	cmp.l	#RGBFB_MaxFormats,d7
	bcc	.end
	
	move.w	(.formats,pc,d7.l*2),d5
	jmp	(.formats,pc,d5.w)

.formats:
	dc.w	.end-.formats
	dc.w	.byte-.formats
	dc.w	.triplebyte-.formats
	dc.w	.triplebyte-.formats
	dc.w	.word16-.formats
	dc.w	.word15-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.end-.formats
	dc.w	.word16-.formats
	dc.w	.word15-.formats
	dc.w	.word16-.formats
	dc.w	.byte-.formats

.triplebyte:
	move.w	d0,d5
	add.w	d0,d0
	add.w	d5,d0
	move.w	d2,d5
	add.w	d2,d2
	add.w	d5,d2
	moveq	#-1,d4
	bra	.all_formats

.word16:
	add.w	d0,d0
	add.w	d2,d2
	moveq	#-1,d4
	bra	.all_formats

.word15:
	add.w	d0,d0
	add.w	d2,d2
	move.l	#$ff7fff7f,d4
	bra	.all_formats

.byte:
	move.b	d4,d5
	swap	d4
	move.b	d5,d4
	lsl.l	#8,d4
	move.b	d5,d4
	swap	d4
	move.b	d5,d4
;	bra	.all_formats

.all_formats:

	move.l	a0,a2

	move.w	(gri_BytesPerRow,a1),d5
	move.l	(gri_Memory,a1),a1

	move.l	gbi_RegisterBase(a2),a0

	muls	d5,d1
	add.l	d1,a1
	add.w	d0,a1
	move.l	a1,d1					; vga operation start address

	sub.l	gbi_MemoryBase(a2),d1

	move.w	#(GDC_BLTMode<<8)|(1<<6),GDCI(a0)	; 8x8 Pattern Copy

	move.w	d5,d0
	SetBLTDstPitch	d0,a0

	cmp.l	#$ffffffff,d4
	bne	.with_mask

	SetBLTROP	ROP_NOTDST,a0
	bra	.no_mask

.with_mask:
	move.l	gbi_MemorySize(a2),d0
	move.l	d0,a1
	add.l	gbi_MemoryBase(a2),a1

	SetBLTSrcStart	d0,a0

	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+

	SetBLTROP	ROP_EOR,a0
.no_mask:

	move.l	d1,a1
	bsr	LargeBlit

.end:
	POP	a2/d2/d4-d5
	rts


***********************************************************
BlitPattern:
***********************************************************
* a0: 	struct BoardInfo
* a1:	struct RenderInfo
* a2:	struct Pattern
* d0: 	WORD X
* d1: 	WORD Y
* d2: 	WORD Width
* d3: 	WORD Height
* d4:	UBYTE Mask
* d7:	ULONG RGBFormat
***********************************************************

	cmp.b	#4,(pat_Size,a2)	; pattern higher than 8 pixels?
	bcc	.fallback

	PUSH	a3			; now check, if we have the right 8 pixels identical to the left ones
	swap	d0
	swap	d1
	move.w	#1,d0
	move.b	(pat_Size,a2),d1
	lsl.w	d1,d0
	subq.w	#1,d0
	move.l	(pat_Memory,a2),a3
.check_loop:
	move.b	(a3)+,d1
	cmp.b	(a3)+,d1
	bne	.cannot_do
	dbra	d0,.check_loop

	swap	d0
	swap	d1
	POP	a3

	jmp	([.formats,pc,d7.l*4])
.formats:
	dc.l	.fallback
	dc.l	.byte
	dc.l	.fallback
	dc.l	.fallback
	dc.l	.word16
	dc.l	.word15
	dc.l	.fallback
	dc.l	.fallback
	dc.l	.fallback
	dc.l	.fallback
	dc.l	.fallback
	dc.l	.fallback
	dc.l	.word16
	dc.l	.word15
	dc.l	.word16
	dc.l	.byte

.cannot_do:
	swap	d0
	swap	d1
	POP	a3
.fallback:
	move.l	(gbi_BlitPatternDefault,a0),-(sp)
	rts

*************************************
.word16:
	PUSH	a3-a4/d2-d7
*************************************
	add.w	d0,d0
	add.w	d2,d2
	moveq	#-1,d4
	move.l	#(1<<4),d7
	bra	.all_formats

*************************************
.word15:
	PUSH	a3-a4/d2-d7
*************************************
	add.w	d0,d0
	add.w	d2,d2
	move.l	#$ff7fff7f,d4
	move.l	#(1<<4),d7
	bra	.all_formats

*************************************
.byte:
	PUSH	a3-a4/d2-d7
*************************************
	move.b	d4,d6
	swap	d4
	move.b	d6,d4
	lsl.l	#8,d4
	move.b	d6,d4
	swap	d4
	move.b	d6,d4
	moveq	#0,d7
;	bra	.all_formats

.all_formats:
	move.l	a0,a3

	move.l	(gbi_MemorySize,a3),a4
	add.l	(gbi_MemoryBase,a3),a4	; where to place the b/w pattern

	movem.w	(pat_Offset,a2),d5/d6

	swap	d0
	swap	d1
	swap	d2

	move.l	(pat_Memory,a2),a0
	move.w	#1,d0
	move.b	(pat_Size,a2),d1
	lsl.w	d1,d0
	subq.w	#1,d0
	and.w	d0,d6
	lea	(a0,d6.w*2),a0
	move.w	d0,d2
	sub.w	d6,d0	
	move.w	#8-1,d6
.pat_loop:
	move.b	(a0),d1
	addq.w	#2,a0
	rol.b	d5,d1
	move.b	d1,(a4)+

	subq.w	#1,d6			; decrement line count
	bmi	.done
	dbra	d0,.pat_loop

	move.l	(pat_Memory,a2),a0
	move.w	d2,d0
	bra	.pat_loop
.done:
	swap	d0
	swap	d1
	swap	d2

	move.l	(gri_Memory,a1),a4
	sub.l	(gbi_MemoryBase,a3),a4
	move.l	(gbi_RegisterBase,a3),a0
	move.w	(gri_BytesPerRow,a1),d6
	mulu	d6,d1
	add.l	d1,a4
	add.w	d0,a4			; destination start address

	move.w	d6,d0
	SetBLTDstPitch	d0,a0		; not modified by blitter op

	btst	#1,(pat_DrawMode,a2)	; COMPLEMENT
	beq	.not_complement

.complement:
	move.w	#(GDC_BLTMode<<8)|(1<<7)|(1<<6),d0		; Color Expand / 8x8 Pattern
	or.w	d7,d0						; maybe 16 Bits
	move.w	d0,GDCI(a0)
	SetBLTROP	ROP_EOR,a0
	move.w	d4,d0
	moveq	#0,d1
	btst	#0,(pat_DrawMode,a2)	; JAM2 ?
	beq	.start_expansion
	move.w	d4,d1
	bra	.start_expansion

.not_complement:

	tst.w	d7					; HiColor ?
	bne	.hicolor
	cmp.b	#$ff,d4
	beq	.no_mask
.mask:

	SetBLTROP	ROP_ONLYDST,a0

	btst	#0,(pat_DrawMode,a2)	; JAM2
	bne	.clear_jam2

.clear_jam1:
	move.w	#(GDC_BLTMode<<8)|(1<<7)|(1<<6),GDCI(a0)	; Color Expand / 8x8 Pattern

	move.l	(gbi_MemorySize,a3),d0
	SetBLTSrcStart	d0,a0

	move.b	d4,d0
	moveq	#0,d1

	btst	#2,(pat_DrawMode,a2)	; INVERSVID
	beq	.clear_jam1_not_invers
	exg	d0,d1
.clear_jam1_not_invers:
	SetGDC	d0,GDC_EnableSetReset		; set fg pen
	SetGDC	d1,GDC_SetReset			; set bg pen

	move.l	a4,a1
	bsr	LargeBlit

	move.b	#GDC_BLTStartStatus,GDCI(a0)
.clear_jam1_wait:
	move.b	GDCD(a0),d0
	btst	#3,d0						; blitter progress status
	bne	.clear_jam1_wait

	move.l	(pat_FgPen,a2),d0
	and.b	d4,d0
	moveq	#0,d1

	bra	.clear_both

.clear_jam2:

	move.l	(gbi_MemorySize,a3),d0
	addq.l	#8,d0
	move.l	d0,a1
	add.l	(gbi_MemoryBase,a3),a1

	SetBLTSrcStart	d0,a0

	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+

	move.w	#(GDC_BLTMode<<8)|(1<<6),GDCI(a0)		; 8x8 Pattern Copy

	move.l	a4,a1
	bsr	LargeBlit

	move.b	#GDC_BLTStartStatus,GDCI(a0)
.clear_jam2_wait:
	move.b	GDCD(a0),d0
	btst	#3,d0						; blitter progress status
	bne	.clear_jam2_wait

	movem.l	(pat_FgPen,a2),d0/d1
	and.b	d4,d0
	and.b	d4,d1

.clear_both:
	SetBLTROP	ROP_OR,a0
	move.w	#(GDC_BLTMode<<8)|(1<<7)|(1<<6),GDCI(a0)	; Color Expand / 8x8 Pattern

	btst	#2,(pat_DrawMode,a2)	; INVERSVID
	beq	.clear_not_invers
	exg	d0,d1
.clear_not_invers:

	SetGDC	d0,GDC_EnableSetReset		; set fg pen
	SetGDC	d1,GDC_SetReset			; set bg pen
	bra	.both

.hicolor:
.no_mask:
	SetBLTROP	ROP_SRC,a0

.start_color_expansion:
	btst	#0,(pat_DrawMode,a2)	; JAM2
	beq	.jam1
.jam2:
	move.w	#(GDC_BLTMode<<8)|(1<<7)|(1<<6),d0		; Color Expand / 8x8 Pattern
	or.w	d7,d0						; maybe 16 Bits
	move.w	d0,GDCI(a0)
	movem.l	(pat_FgPen,a2),d0/d1
	and.w	d4,d0
	and.w	d4,d1
	bra	.start_expansion
.jam1:
	move.w	#(GDC_BLTMode<<8)|(1<<7)|(1<<3)|(1<<6),d0	; Transparency / Color Expand / 8x8 Pattern
	or.w	d7,d0						; maybe 16 Bits
	move.w	d0,GDCI(a0)
	move.l	(pat_FgPen,a2),d0
	and.w	d4,d0
	move.w	d0,d1
	not.w	d1
	not.w	d4
	SetGDC	d4,GDC_BLTTranspColorMaskHigh	; compare all bits
	lsr.w	#8,d4
	SetGDC	d4,GDC_BLTTranspColorMaskLow	; compare all bits
	move.w	d1,d4
	SetGDC	d4,GDC_BLTTranspColorHigh	; set mask pen
	tst.w	d7				; HiColor ?
	beq	.duplicate	
	lsr.w	#8,d4
.duplicate:
	SetGDC	d4,GDC_BLTTranspColorLow	; set mask pen

.start_expansion:
	btst	#2,(pat_DrawMode,a2)	; INVERSVID
	beq	.not_invers
	exg	d0,d1
.not_invers:

	tst.w	d7				; HiColor ?
	bne	.pixels_16_bits
	SetGDC	d0,GDC_EnableSetReset		; set fg pen
	SetGDC	d1,GDC_SetReset			; set bg pen
	bra	.both
.pixels_16_bits:
	SetGDC	d0,GDC_PixelFgColorHigh
	lsr.w	#8,d0
	SetGDC	d0,GDC_EnableSetReset		; set fg pen
	SetGDC	d1,GDC_PixelBgColorHigh
	lsr.w	#8,d1
	SetGDC	d1,GDC_SetReset			; set bg pen

.both:
	move.l	(gbi_MemorySize,a3),d0
	SetBLTSrcStart	d0,a0

	move.l	a4,a1
	bsr	LargeBlit

	move.b	#GDC_BLTStartStatus,GDCI(a0)
.wait:	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.wait

.end:
	SetGDC	#0,GDC_EnableSetReset		; set fg pen

*************************************
	POP	a3-a4/d2-d7
	rts


***********************************************************
BlitTemplate:
***********************************************************
* a0: 	struct BoardInfo
* a1:	struct RenderInfo
* a2:	struct Template
* d0: 	WORD X
* d1: 	WORD Y
* d2: 	WORD Width
* d3: 	WORD Height
* d4:	UBYTE Mask
* d7:	ULONG RGBFormat
***********************************************************

	jmp	([.formats,pc,d7.l*4])
.formats:
	dc.l	.fallback
	dc.l	.byte
	dc.l	.fallback
	dc.l	.fallback
	dc.l	.word16
	dc.l	.word15
	dc.l	.fallback
	dc.l	.fallback
	dc.l	.fallback
	dc.l	.fallback
	dc.l	.fallback
	dc.l	.fallback
	dc.l	.word16
	dc.l	.word15
	dc.l	.word16
	dc.l	.byte

.fallback:
	move.l	(gbi_BlitTemplateDefault,a0),-(sp)
	rts

*************************************
.word16:
	cmp.w	#MAX_BLTWIDTH/2,d2
	bgt	.fallback
	PUSH	a3-a4/d2-d7
*************************************
	move.w	d2,d5
	add.w	d0,d0
	add.w	d2,d2
	moveq	#-1,d4
	move.l	#(1<<4),d7
	bra	.all_formats

*************************************
.word15:
	cmp.w	#MAX_BLTWIDTH/2,d2
	bgt	.fallback
	PUSH	a3-a4/d2-d7
*************************************
	move.w	d2,d5
	add.w	d0,d0
	add.w	d2,d2
	move.l	#$ff7fff7f,d4
	move.l	#(1<<4),d7
	bra	.all_formats

*************************************
.byte:
	cmp.w	#MAX_BLTWIDTH,d2
	bgt	.fallback
	PUSH	a3-a4/d2-d7
*************************************
	move.w	d2,d5
	move.b	d4,d6
	swap	d4
	move.b	d6,d4
	lsl.l	#8,d4
	move.b	d6,d4
	swap	d4
	move.b	d6,d4
	moveq	#0,d7
;	bra	.all_formats

.all_formats:
	move.l	a0,a3

	move.l	(gri_Memory,a1),a4
	sub.l	(gbi_MemoryBase,a3),a4
	move.l	(gbi_RegisterBase,a3),a0
	move.w	(gri_BytesPerRow,a1),d6
	mulu	d6,d1
	add.l	d1,a4
	add.w	d0,a4			; destination start address

	move.w	d2,d0
	SetBLTWidth	d0,a0		; not modified by blitter op

	move.w	d6,d0
	SetBLTDstPitch	d0,a0		; not modified by blitter op

	add.w	#7,d5
	and.w	#~7,d5
	lsr.w	#3,d5
	move.w	d5,d2
	move.w	(tmp_BytesPerRow,a2),d5
	sub.w	d2,d5			; modulo-remainder for amiga planes

	btst	#1,(tmp_DrawMode,a2)	; COMPLEMENT
	beq	.not_complement

.complement:
	move.w	#(GDC_BLTMode<<8)|(1<<7)|(1<<2),d0		; Color Expand / System Mem Src
	or.w	d7,d0						; maybe 16 Bits
	move.w	d0,GDCI(a0)
	SetBLTROP	ROP_EOR,a0
	move.w	d4,d0
	moveq	#0,d1
	btst	#0,(tmp_DrawMode,a2)	; JAM2 ?
	beq	.start_expansion
	move.w	d4,d1
	bra	.start_expansion

.not_complement:

	tst.w	d7					; HiColor ?
	bne	.hicolor
	cmp.b	#$ff,d4
	beq	.no_mask
.mask:

	move.l	a4,d0
	SetBLTDstStart	d0,a0

	move.w	d3,d0
	SetBLTHeight	d0,a0

	SetBLTROP	ROP_ONLYDST,a0

	btst	#0,(tmp_DrawMode,a2)	; JAM2
	bne	.clear_jam2

.clear_jam1:
	move.w	#(GDC_BLTMode<<8)|(1<<7)|(1<<2),GDCI(a0)	; Color Expand / System Mem Src

	move.w	#GDC_BLTSrcStartLow<<8,GDCI(a0)
	move.w	#GDC_BLTSrcStartMiddle<<8,GDCI(a0)
	move.w	#GDC_BLTSrcStartHigh<<8,GDCI(a0)

	move.b	d4,d0
	moveq	#0,d1

	btst	#2,(tmp_DrawMode,a2)	; INVERSVID
	beq	.clear_jam1_not_invers
	exg	d0,d1
.clear_jam1_not_invers:
	SetGDC	d0,GDC_EnableSetReset		; set fg pen
	SetGDC	d1,GDC_SetReset			; set bg pen

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	nop

	move.l	(gbi_MemoryBase,a3),a1
	bsr	ExpansionWrite

.clear_jam1_wait:
	move.b	GDCD(a0),d0
	btst	#3,d0						; blitter progress status
	bne	.clear_jam1_wait

	move.l	(tmp_FgPen,a2),d0
	and.b	d4,d0
	moveq	#0,d1

	bra	.clear_both

.clear_jam2:

	move.l	(gbi_MemorySize,a3),d0
	move.l	d0,a1
	add.l	(gbi_MemoryBase,a3),a1

	SetBLTSrcStart	d0,a0

	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+
	move.l	d4,(a1)+

	move.w	#(GDC_BLTMode<<8)|(1<<6),GDCI(a0)		; 8x8 Pattern Copy

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	nop

.clear_jam2_wait:
	move.b	GDCD(a0),d0
	btst	#3,d0						; blitter progress status
	bne	.clear_jam2_wait

	movem.l	(tmp_FgPen,a2),d0/d1
	and.b	d4,d0
	and.b	d4,d1

.clear_both:
	SetBLTROP	ROP_OR,a0
	move.w	#(GDC_BLTMode<<8)|(1<<7)|(1<<2),GDCI(a0)	; Color Expand / System Mem Src

	btst	#2,(tmp_DrawMode,a2)	; INVERSVID
	beq	.clear_not_invers
	exg	d0,d1
.clear_not_invers:

	SetGDC	d0,GDC_EnableSetReset		; set fg pen
	SetGDC	d1,GDC_SetReset			; set bg pen
	bra	.both

.hicolor:
.no_mask:
	SetBLTROP	ROP_SRC,a0

.start_color_expansion:
	btst	#0,(tmp_DrawMode,a2)	; JAM2
	beq	.jam1
.jam2:
	move.w	#(GDC_BLTMode<<8)|(1<<7)|(1<<2),d0		; Color Expand / System Mem Src
	or.w	d7,d0						; maybe 16 Bits
	move.w	d0,GDCI(a0)
	movem.l	(tmp_FgPen,a2),d0/d1
	and.w	d4,d0
	and.w	d4,d1
	bra	.start_expansion
.jam1:
	move.w	#(GDC_BLTMode<<8)|(1<<7)|(1<<3)|(1<<2),d0	; Transparency / Color Expand / System Mem Src
	or.w	d7,d0						; maybe 16 Bits
	move.w	d0,GDCI(a0)
	move.l	(tmp_FgPen,a2),d0
	and.w	d4,d0
	move.w	d0,d1
	not.w	d1
	not.w	d4
	SetGDC	d4,GDC_BLTTranspColorMaskHigh	; compare all bits
	lsr.w	#8,d4
	SetGDC	d4,GDC_BLTTranspColorMaskLow	; compare all bits
	move.w	d1,d4
	SetGDC	d4,GDC_BLTTranspColorHigh	; set mask pen
	tst.w	d7				; HiColor ?
	beq	.duplicate	
	lsr.w	#8,d4
.duplicate:
	SetGDC	d4,GDC_BLTTranspColorLow	; set mask pen

.start_expansion:
	btst	#2,(tmp_DrawMode,a2)	; INVERSVID
	beq	.not_invers
	exg	d0,d1
.not_invers:

	tst.w	d7				; HiColor ?
	bne	.pixels_16_bits
	SetGDC	d0,GDC_EnableSetReset		; set fg pen
	SetGDC	d1,GDC_SetReset			; set bg pen
	bra	.both
.pixels_16_bits:
	SetGDC	d0,GDC_PixelFgColorHigh
	lsr.w	#8,d0
	SetGDC	d0,GDC_EnableSetReset		; set fg pen
	SetGDC	d1,GDC_PixelBgColorHigh
	lsr.w	#8,d1
	SetGDC	d1,GDC_SetReset			; set bg pen

.both:
	move.l	a4,d0
	SetBLTDstStart	d0,a0

	move.w	d3,d0
	SetBLTHeight	d0,a0

	move.w	#GDC_BLTSrcStartLow<<8,GDCI(a0)
	move.w	#GDC_BLTSrcStartMiddle<<8,GDCI(a0)
	move.w	#GDC_BLTSrcStartHigh<<8,GDCI(a0)

	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
	nop

	move.l	(gbi_MemoryBase,a3),a1
	bsr	ExpansionWrite

.end:
	SetGDC	#0,GDC_EnableSetReset		; set fg pen

*************************************
	POP	a3-a4/d2-d7
	rts


ExpansionWrite:
;	a2:	bitdata address
;	a1:	io address
;	d2:	bytes per row
;	d3:	rows
;	d4:	displacement
;	d5:	bitdata pitch

	PUSH	d2-d4/d6-d7/a2

	moveq	#0,d4
	move.b	(tmp_XOffset,a2),d4
	move.l	(tmp_Memory,a2),a2
	moveq	#0,d7			; flags
	lsr.w	#1,d2			; words for xloop
	bcs	.odd			; odd/even?

;---------------------------------------; even number of bytes per row
.even:
	move.w	d2,d7			; words per row
	and.w	d3,d7			; number of rows
	and.w	#1,d7			; odd number of words if both odd
	subq.w	#1,d3			; for yloop
	subq.w	#1,d2			; for xloop
	tst.b	d4			; src bit offset
	beq	.even_aligned

	neg.w	d4
	add.w	#16,d4
.even_unaligned_y_loop:
	move.w	d2,d0			; loop counter

.even_unaligned_x_loop:
	move.l	(a2),d1
	lsr.l	d4,d1
	addq.w	#2,a2
	move.w	d1,(a1)
	dbra	d0,.even_unaligned_x_loop
	add.w	d5,a2			; add modulo-remainder for amiga planes
	dbra	d3,.even_unaligned_y_loop
	bra	.almostdone

.even_aligned:
.even_aligned_y_loop:
	move.w	d2,d0			; loop counter

.even_aligned_x_loop:
	move.w	(a2)+,(a1)
	dbra	d0,.even_aligned_x_loop
	add.w	d5,a2			; add modulo-remainder for amiga planes
	dbra	d3,.even_aligned_y_loop
	bra	.almostdone

;---------------------------------------; odd number of bytes per row
.odd:
	subq.w	#1,d3			; for yloop
	subq.w	#1,d2			; for xloop
	tst.b	d4			; src bit offset
	beq	.odd_aligned

	neg.w	d4
	add.w	#16,d4
.odd_unaligned:
.odd_unaligned_y_loop:
	move.w	d2,d0			; loop counter
	bmi	.odd_unaligned_one_byte1
.odd_unaligned_x_loop1:
	move.l	(a2),d1
	lsr.l	d4,d1
	addq.w	#2,a2
	move.w	d1,(a1)
	not.w	d7
	dbra	d0,.odd_unaligned_x_loop1

.odd_unaligned_one_byte1:
	move.l	(a2),d1
	lsr.l	d4,d1
	addq.w	#1,a2

	subq.w	#1,d3
	bmi	.odd_unaligned_extra_word

	add.w	d5,a2			; add modulo-remainder for amiga planes
	move.l	(a2),d6
	lsr.l	d4,d6
	lsr.l	#8,d6
	addq.w	#1,a2
	move.b	d6,d1
	move.w	d1,(a1)
	not.w	d7

	move.w	d2,d0			; loop counter
	bmi	.odd_unaligned_one_byte2
.odd_unaligned_x_loop2:
	move.l	(a2),d1
	lsr.l	d4,d1
	addq.w	#2,a2
	move.w	d1,(a1)
	not.w	d7
	dbra	d0,.odd_unaligned_x_loop2

.odd_unaligned_one_byte2:
	add.w	d5,a2			; add modulo-remainder for amiga planes
	dbra	d3,.odd_unaligned_y_loop
	bra	.almostdone

.odd_unaligned_extra_word:
	move.w	d1,(a1)
	not.w	d7
	bra	.almostdone

.odd_aligned:
.odd_aligned_y_loop:
	move.w	d2,d0			; loop counter
	bmi	.odd_aligned_one_byte1
.odd_aligned_x_loop1:
	move.w	(a2)+,(a1)
	not.w	d7
	dbra	d0,.odd_aligned_x_loop1

.odd_aligned_one_byte1:
	move.b	(a2)+,d1			; accumulate byte
	lsl.w	#8,d1

	subq.w	#1,d3
	bmi	.odd_aligned_extra_word

	add.w	d5,a2			; add modulo-remainder for amiga planes
	move.b	(a2)+,d1			; accumulated write, high byte is already set
	move.w	d1,(a1)
	not.w	d7

	move.w	d2,d0			; loop counter
	bmi	.odd_aligned_one_byte2
.odd_aligned_x_loop2:
	move.w	(a2)+,(a1)
	not.w	d7
	dbra	d0,.odd_aligned_x_loop2

.odd_aligned_one_byte2:
	add.w	d5,a2			; add modulo-remainder for amiga planes
	dbra	d3,.odd_aligned_y_loop
	bra	.almostdone

.odd_aligned_extra_word:
	move.w	d1,(a1)
	not.w	d7

.almostdone:
	tst.w	d7
	beq	.done
	move.w	#0,(a1)			; if odd number of words written
.done:
	POP	d2-d4/d6-d7/a2
	rts

*************************************************************************************

DepthMask_table:
	dc.b	%00000000
	dc.b	%00000001
	dc.b	%00000011
	dc.b	%00000111
	dc.b	%00001111
	dc.b	%00011111
	dc.b	%00111111
	dc.b	%01111111
	dc.b	%11111111,0,0,0

	BITDEF	OP,CLEARBEFORE,15
	BITDEF	OP,INVERTBEFORE,14
	BITDEF	OP,INVERTAFTER,13
	BITDEF	OP,AND,12

Opcode_table:
	dc.w	-1				; ROP_FALSE (-> FillRect)
	dc.w	ROP_OR|OPF_INVERTAFTER		; ROP_NOR
	dc.w	ROP_ONLYDST			; ROP_ONLYDST (~s /\ d)
	dc.w	ROP_NOTONLYSRC|OPF_AND|OPF_CLEARBEFORE	; ROP_NOTSRC
	dc.w	ROP_AND|OPF_AND|OPF_INVERTBEFORE	; ROP_ONLYSRC (s /\ ~d)
	dc.w	-1				; ROP_NOTDST (-> InvertRect)
	dc.w	ROP_EOR				; ROP_EOR
	dc.w	ROP_AND|OPF_AND|OPF_INVERTAFTER	; ROP_NAND
	dc.w	ROP_AND|OPF_AND			; ROP_AND
	dc.w	ROP_EOR|OPF_INVERTAFTER		; ROP_NEOR
	dc.w	-1				; ROP_DST (nop)
	dc.w	ROP_NOTONLYSRC|OPF_AND		; ROP_NOTONLYSRC (~s \/ d)
	dc.w	ROP_OR|OPF_CLEARBEFORE		; ROP_SRC
	dc.w	ROP_OR|OPF_INVERTBEFORE		; ROP_NOTONLYDST (s \/ ~d)
	dc.w	ROP_OR				; ROP_OR
	dc.w	-1				; ROP_TRUE (-> FillRect)

*************************************************************************************
BlitPlanar2Chunky:
* void __asm BlitPlanar2Chunky(
* register __a0 struct BoardInfo *,
* register __a1 struct BitMap *,
* register __a2 struct RenderInfo *,
* register __d0 SHORT SrcX,
* register __d1 SHORT SrcY,
* register __d2 SHORT DstX,
* register __d3 SHORT DstY,
* register __d4 SHORT SizeX,
* register __d5 SHORT SizeY,
* register __d6 UBYTE MinTerm);
* register __d7 UBYTE Mask);
*************************************************************************************

	cmp.w	#MAX_BLTWIDTH,d4
	ble	.width_ok
	move.l	(gbi_BlitPlanar2ChunkyDefault,a0),-(sp)
	rts

.width_ok:
	PUSH	d2-d7/a2-a6

	subq.w	#1,d5				; height - 1
	swap	d5
	move.w	d4,d5

	exg	d0,d2
	exg	d1,d3
	exg	d0,d1

	move.l	a0,a3				; BoardInfo
	move.l	a1,a5				; BitMap
	move.l	(gbi_MemoryBase,a3),a4
	move.l	a4,a1
	add.l	(gbi_MemorySize,a3),a1		; temporary Memory
	move.l	(gbi_RegisterBase,a3),a0	; RegisterBase
	move.l	(gri_Memory,a2),a6		; Screen Memory
	suba.l	a4,a6
	add.w	d1,a6				; dstx
	move.w	(gri_BytesPerRow,a2),d4
	mulu	d4,d0
	add.l	d0,a6				; Blit Start Address
	swap	d4

	move.b	d7,d0
	lsl.w	#8,d7
	move.b	d0,d7
	move.w	d7,d0
	swap	d7
	move.w	d0,d7
	move.l	d7,(a1)+			; write clear mask to vga memory
	move.l	d7,(a1)+
	move.l	d7,(a1)+
	move.l	d7,(a1)+
	move.l	d7,(a1)+
	move.l	d7,(a1)+
	move.l	d7,(a1)+
	move.l	d7,(a1)+
	move.l	d7,(a1)+
	move.l	d7,(a1)+
	move.l	d7,(a1)+
	move.l	d7,(a1)+
	move.l	d7,(a1)+
	move.l	d7,(a1)+
	move.l	d7,(a1)+
	move.l	d7,(a1)+

	swap	d7
	ext.w	d6
	move.w	(Opcode_table,pc,d6.w*2),d7
	cmp.w	#-1,d7
	beq	.error
	swap	d7

	moveq	#0,d0
	move.b	(bm_Depth,a5),d0
	and.b	(DepthMask_table,pc,d0.w),d7
	lsl.w	#8,d7

	move.w	#(GDC_BLTDstPitchLow<<8),GDCI(a0)	; pitch = 0
	move.w	#(GDC_BLTDstPitchHigh<<8),GDCI(a0)

	move.w	d5,d0
	SetBLTWidth	d0,a0
	subq.w	#1,d5				; width - 1

	move.w	(bm_BytesPerRow,a5),d6
	move.w	d2,d4
	and.w	#15,d4
	neg.w	d4
	add.w	#16,d4
	add.w	d2,d5
	lsr.w	#4,d2
	lsr.w	#4,d5
	sub.w	d2,d5			; width in words - 1
	muls	d6,d3			; by bm_BytesPerRow
	move.l	d3,a2
	add.w	d2,a2			; add byte offset in line
	add.w	d2,a2			; save bitmap start offset

	lea	(bm_Planes,a5),a5
	move.l	a5,d1
	swap	d5
	swap	d4
.y_loop:

	move.l	a6,d0
	SetBLTDstStart	d0,a0

	btst	#OPB_CLEARBEFORE+16,d7
	bne	.clear_before

	btst	#OPB_INVERTBEFORE+16,d7
	beq	.nothing_before
	
	SetBLTROP	ROP_EOR,a0
	bra	.do_before
.clear_before:
	SetBLTROP	ROP_ONLYDST,a0
.do_before:
	move.l	(gbi_MemorySize,a3),d0
	SetBLTSrcStart	d0,a0

	move.w	#(GDC_BLTHeightLow<<8),GDCI(a0)		; height = 1
	move.w	#(GDC_BLTHeightHigh<<8),GDCI(a0)

	move.w	#(GDC_BLTMode<<8)|(1<<6),GDCI(a0)		; 8x8 Pattern Copy
	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
.before_wait_mask:
	move.b	GDCD(a0),d0
	btst	#3,d0						; blitter progress status
	bne	.before_wait_mask

.nothing_before:
	swap	d7
	SetGDC	d7,GDC_BLTRasterOperation
	swap	d7
	move.w	#(GDC_BLTMode<<8)|(1<<7)|(1<<2),GDCI(a0)	; Color Expand

	btst	#OPB_AND+16,d7
	beq	.no_and
	move.l	#$fffeffff,d3			; initial mask
	bra	.no_or
.no_and:
	move.l	#$00000001,d3			; initial mask
.no_or:

	move.w	d7,d0
	lsr.w	#8,d0
	move.b	d0,d7

	swap	d6
	swap	d4

.mask_loop:
	lsr.b	#1,d7
	bcc	.plane_not_involved

.plane_involved:
	swap	d5
	move.w	d5,d6			; words to do

	swap	d3
	SetGDC	d3,GDC_SetReset		; set plane mask
	swap	d3
	SetGDC	d3,GDC_EnableSetReset	; set plane mask

	move.w	#(GDC_BLTHeightLow<<8),GDCI(a0)		; height = 1
	move.w	#(GDC_BLTHeightHigh<<8),GDCI(a0)

	move.w	#GDC_BLTSrcStartLow<<8,GDCI(a0)
	move.w	#GDC_BLTSrcStartMiddle<<8,GDCI(a0)
	move.w	#GDC_BLTSrcStartHigh<<8,GDCI(a0)
	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT

	move.l	(a5),d0
	beq	.x_loop_dummy
	cmp.l	#-1,d0
	beq	.x_loop_dummy

	move.l	d0,a1			; planeptr
	add.l	a2,a1			; + offset (start in bitmap)

.x_loop:
	move.l	(a1),d2
	lsr.l	d4,d2
	addq.w	#2,a1
	move.w	d2,(a4)			; blitter wait for data at start of display memory
	dbra	d6,.x_loop
	btst	#0,d5
	bne	.even_words
	bra	.odd_words

.x_loop_dummy:
	move.w	d0,(a4)
	dbra	d6,.x_loop_dummy
	btst	#0,d5
	bne	.even_words
.odd_words:
	move.w	d0,(a4)
.even_words:

	swap	d5			; height
.blitter_wait_loop:					; a Must !!!
	move.b	GDCD(a0),d0
	btst	#3,d0				; blitter progress status
	bne	.blitter_wait_loop

.plane_not_involved:
	rol.l	#1,d3			; shift plane mask
	add.w	#4,a5			; next planeptr
	tst.b	d7			; any more planes ?
	bne	.mask_loop

	btst	#OPB_INVERTAFTER+16,d7
	beq	.no_invert_after

	move.l	(gbi_MemorySize,a3),d0
	SetBLTSrcStart	d0,a0

	move.w	#(GDC_BLTHeightLow<<8),GDCI(a0)		; height = 1
	move.w	#(GDC_BLTHeightHigh<<8),GDCI(a0)

	SetBLTROP	ROP_EOR,a0
	move.w	#(GDC_BLTMode<<8)|(1<<6),GDCI(a0)		; 8x8 Pattern Copy
	nop
	move.w	#(GDC_BLTStartStatus<<8)|(11),GDCI(a0)	; start BLT
.after_wait_mask:
	move.b	GDCD(a0),d0
	btst	#3,d0						; blitter progress status
	bne	.after_wait_mask

.no_invert_after:
	swap	d6
	swap	d4

	move.l	d1,a5			; restore planeptr to planeptr 0
	add.w	d6,a2			; add modulo-remainder for amiga planes offset
	add.w	d4,a6			; add modulo-remainder for chunky bitmap address
	dbra	d5,.y_loop

.finished:
	SetGDC	#0,GDC_EnableSetReset		; set plane mask
.error:
	POP	d2-d7/a2-a6
	rts

*************************************************************************************
SetDisplay:
* a0:	struct BoardInfo
* d0:	BOOL state
*************************************************************************************
	move.l	gbi_RegisterBase(a0),a0
	not.b	d0
	and.w	#1,d0
	lsl.b	#5,d0
	move.b	#TS_TSMode,TSI(a0)
	move.b	TSD(a0),d1
	and.b	#%11011111,d1
	or.b	d0,d1	
	move.b	d1,TSD(a0)
	rts

****************************************************************************************
SetMemoryMode:
* a0:	struct BoardInfo
* d7:	RGBFTYPE RGBFormat
****************************************************************************************
	cmp.l	#RGBFB_PLANAR,d7
	sne	d0
	and.w	#1,d0
	lsl.b	#3,d0
	or.w	#(TS_MemoryMode<<8)|$06,d0	; enable/disable chain 4
	move.w	d0,([gbi_RegisterBase,a0],TSI)
	nop
	rts

;	PUSH	a0/d0-d1
;	move.l	(gbi_RegisterBase,a0),a0

;	and.b	#1,d0
;	move.b	#TS_ExtendedSequencerMode,TSI(a0)
;	move.b	TSD(a0),d1
;	and.b	#%11111110,d1
;	or.b	d0,d1
;	move.b	d1,TSD(a0)

;	move.b	#TS_MemoryMode,TSI(a0)
;	move.b	d0,d1
;	lsl.b	#3,d0
;	or.b	#$06,d0				; enable/disable chain 4
;	move.b	d0,TSD(a0)

;	POP	a0/d0-d1
;	rts

***************************************************************************************
SetWriteMask:
* a0:	struct BoardInfo
* d0:	UBYTE mask
***************************************************************************************
	PUSH	a0
	move.l	gbi_RegisterBase(a0),a0

	move.b	#TS_WritePlaneMask,TSI(a0)
	move.b	d0,TSD(a0)
	nop
	POP	a0
	rts

***************************************************************************************
SetReadPlane:
* a0:	struct BoardInfo
* d0:	UBYTE Plane
***************************************************************************************
	and.b	#3,d0
	move.l	gbi_RegisterBase(a0),a0

	move.b	#GDC_ReadPlaneSelect,GDCI(a0)
	move.b	d0,GDCD(a0)
	nop
	rts

***************************************************************************************
SetClearMask:
* a0:	struct BoardInfo
* d0:	UBYTE mask
***************************************************************************************
	move.b	d0,gbi_ClearMask(a0)
	move.l	gbi_RegisterBase(a0),a0

	SetGDC	#$00,GDC_SetReset

	move.b	#GDC_EnableSetReset,GDCI(a0)
	move.b	d0,GDCD(a0)
	nop
	rts

***************************************************************************************
WaitVerticalSync:
* a0:	struct BoardInfo
* d0:	BOOL End
***************************************************************************************

	move.l	gbi_RegisterBase(a0),a0
	lea	INPUTSTATUS1(a0),a0
	tst.b	d0
	beq	.sync_start

.sync_end:
	moveq	#-1,d1
.wait_end:
	move.b	(a0),d0
	and.b	#(1<<3),d0
	beq	.done
	subq.l	#1,d1
	bne	.wait_end
	rts

.sync_start:
	moveq	#-1,d1
.wait_start:
	move.b	(a0),d0
	and.b	#(1<<3),d0
	bne	.done
	subq.l	#1,d1
	bne	.wait_start
.done:
	rts

*****************************************************************
SetClock:
* a0:	struct BoardInfo
*****************************************************************
	move.l	gbi_ModeInfo(a0),a1
	move.l	gbi_RegisterBase(a0),a0
	moveq	#0,d1

.numerator:
	move.b	gmi_Numerator(a1),d1

	move.b	#TS_VCLK3Numerator,TSI(a0)
	move.b	d1,TSD(a0)

.denominator:
	move.b	gmi_Denominator(a1),d1
	lsl.b	#1,d1

	move.b	#TS_VCLK3Denominator,TSI(a0)
	move.b	d1,TSD(a0)

	rts

*****************************************************************
ResolvePixelClock:
* a0:	struct BoardInfo
* a1:	struct ModeInfo
* d0.l:	pixelclock
* d7:	RGBFormat
*****************************************************************
	PUSH	d2-d3
	move.l	#0,d2
	move.b	(.clockcount_array,pc,d7.l),d2

	move.l	(gbi_Flags,a0),d1
	btst	#BIB_OVERCLOCK,d1
	beq	.normal
	add.w	#OVERCLOCKS,d2
.normal:

	and.b	#~GMF_DOUBLEVERTICAL,gmi_Flags(a1)
	cmp.w	#1024,gmi_VerTotal(a1)
	blt	.vertical_value_ok
	or.b	#GMF_DOUBLEVERTICAL,gmi_Flags(a1)
.vertical_value_ok:

	and.b	#~GMF_DOUBLECLOCK,gmi_Flags(a1)

	move.l	d0,d1
	moveq	#0,d3
	move.b	(.clock_multipliers,pc,d7.l),d3
	jmp	([.table-4,pc,d3.l*4])
.table:
	dc.l	.one,.two,.three,.four
.three:
	add.l	d1,d1	
	add.l	d0,d1	
	bra	.common

.four:
	add.l	d1,d1	
.two:							; 16 bit data at pixel rate
	add.l	d1,d1	
.one:
.common:
	lea	PixelClocks(pc),a0
	moveq	#0,d0
.loop:
	cmp.l	(a0)+,d1
	beq	.found
	blt	.adjust
	addq.l	#1,d0
	cmp.l	d2,d0
	blt	.loop

	subq.l	#1,d0
	move.l	-4(a0),d1
	bra	.found

.adjust:
	tst.l	d0
	beq	.first
	move.l	-4(a0),d2
	add.l	-8(a0),d2
	sub.l	d1,d2
	sub.l	d1,d2
	bmi	.less
.first:
	move.l	-8(a0),d1
	subq.l	#1,d0
	bra	.found
.less:
	move.l	-4(a0),d1
.found:
	divu.l	d3,d1
	move.l	d1,gmi_PixelClock(a1)
	move.b	Numerators(pc,d0.l),gmi_Numerator(a1)
	move.b	Denominators(pc,d0.l),gmi_Denominator(a1)
.end:
	POP	d2-d3
	rts

.clock_multipliers:
	dc.b	1,1,3,3,1,1,4,4,4,4,1,1,1,1,1,1
.clockcount_array:
	dc.b	NUMCLOCKS, NUMCLOCKS, NUMCLOCKS, NUMCLOCKS, NUMHICOLORCLOCKS, NUMHICOLORCLOCKS, 0, 0, 0, 0
	dc.b	NUMHICOLORCLOCKS, NUMHICOLORCLOCKS, NUMHICOLORCLOCKS, NUMHICOLORCLOCKS, 0, 0

	cnop	0,4

*****************************************************************
GetPixelClock:
* a0:	struct BoardInfo
* a1:	struct ModeInfo
* d0.l:	index
* d7:	RGBFormat
*****************************************************************
	lea	(PixelClocks,pc),a0
	move.l	(a0,d0.l*4),d0
	moveq	#0,d1
	move.b	(.clock_dividers,pc,d7.l),d1
	divu.l	d1,d0
	rts
.clock_dividers:
	dc.b	1,1,3,3,1,1,4,4,4,4,1,1,1,1,1,1

;-----------------------------------------------

NUMCLOCKS		EQU	169
OVERCLOCKS		EQU	20
NUMHICOLORCLOCKS	EQU	74

Numerators:
	dc.b	 13, 10, 15, 11, 14, 16, 17,  6, 21, 11
	dc.b	 10, 12, 26, 21, 20, 17, 30, 30, 22, 26
	dc.b	 19, 30, 19, 11, 39, 31, 41, 30,  7, 43
	dc.b	 22, 36, 43, 44, 45, 41, 52, 53, 49, 41
	dc.b	 49, 37, 49, 48, 45, 44, 61, 35, 44, 49
	dc.b	 39, 33, 38, 59, 53, 68, 19, 53, 44, 62
	dc.b	 78, 51, 31, 55, 69, 78, 79, 80, 81, 82
	dc.b	 63, 87, 88, 92, 63, 79, 83, 87, 44, 89
	dc.b	 90, 65, 82, 73, 57,105, 89, 83, 98, 67
	dc.b	 57, 72, 69, 66,111, 71, 83,118, 73, 62
	dc.b	 90, 75,123, 52, 81, 94,103, 54, 88, 93
	dc.b	 98,116,117, 96, 22,102, 76, 36, 59,119
	dc.b	106, 65,117, 33,114,110,106, 68, 44, 64
	dc.b	119, 90,121, 81,102, 77, 31, 52,110,116
	dc.b	 69,123, 43, 92,109,126,127,111, 95, 90
	dc.b	113, 74,126,121, 29, 70, 88, 59, 95,125
	dc.b	 60,121, 79,110, 43, 68, 87,125, 88,120
	dc.b	 89,115, 90, 84, 91,111,105, 66, 73, 80
	dc.b	114,101,122,109, 89,117, 83,125, 98

Denominators:
	dc.b	31,22,31,21,25,27,27, 9,30,15
	dc.b	13,15,31,24,22,18,31,30,21,24
	dc.b	17,26,16, 9,31,24,31,22, 5,30
	dc.b	15,24,28,28,28,25,31,31,28,23
	dc.b	27,20,26,25,23,22,30,17,21,23
	dc.b	18,15,17,26,23,29, 8,22,18,25
	dc.b	31,20,12,21,26,29,29,29,29,29
	dc.b	22,30,30,31,21,26,27,28,14,28
	dc.b	28,20,25,22,17,31,26,24,28,19
	dc.b	16,20,19,18,30,19,22,31,19,16
	dc.b	23,19,31,13,20,23,25,13,21,22
	dc.b	23,27,27,22, 5,23,17, 8,13,26
	dc.b	23,14,25, 7,24,23,22,14, 9,13
	dc.b	24,18,24,16,20,15, 6,10,21,22
	dc.b	13,23, 8,17,20,23,23,20,17,16
	dc.b	20,13,22,21, 5,12,15,10,16,21
	dc.b	10,20,13,18, 7,11,14,20,14,19
	dc.b	14,18,14,13,14,17,16,10,11,12
	dc.b	17,15,18,16,13,17,12,18,14

PixelClocks:
	dc.l	  6004398,  6508263,  6928151,  7499999,  8018180,  8484847,  9015150,  9545453, 10022726, 10499998
	dc.l	 11013984, 11454544, 12008796, 12528407, 13016527, 13522725, 13856303, 14318180, 14999998, 15511361
	dc.l	 16002671, 16520976, 17002838, 17499997, 18013194, 18494315, 18936947, 19524790, 20045452, 20522724
	dc.l	 20999997, 21477270, 21988633, 22499997, 23011360, 23481815, 24017592, 24479469, 25056815, 25523712
	dc.l	 25984845, 26488633, 26984262, 27490905, 28013830, 28636360, 29113632, 29478605, 29999996, 30503948
	dc.l	 31022723, 31499996, 32005343, 32491254, 32994066, 33573663, 34005677, 34493797, 34999995, 35509086
	dc.l	 36026388, 36511359, 36988631, 37499995, 37998246, 38510966, 39004697, 39498427, 39992157, 40485888
	dc.l	 41002060, 41522722, 41999994, 42492663, 42954540, 43505239, 44015145, 44488630, 44999994, 45511357
	dc.l	 46022721, 46534085, 46963630, 47510324, 48008015, 48497061, 49012231, 49517039, 50113630, 50490424
	dc.l	 51008516, 51545448, 51997601, 52499993, 52977266, 53504777, 54018588, 54501459, 55011954, 55482947
	dc.l	 56027660, 56519131, 56810843, 57272720, 57988629, 58517779, 58990901, 59475516, 59999992, 60526851
	dc.l	 61007897, 61515143, 62045446, 62479330, 62999992, 63498015, 64010687, 64431810, 64982509, 65533208
	dc.l	 65988133, 66477264, 67009082, 67499991, 68011355, 68478252, 68987594, 69545445, 69999991, 70489501
	dc.l	 70994309, 71590900, 72187490, 72485786, 73022718, 73499990, 73977263, 74454536, 74999990, 75495858
	dc.l	 75996493, 76571136, 76960217, 77486621, 78034081, 78438725, 79061254, 79465899, 80013358, 80539762
	dc.l	 80897717, 81503486, 82004121, 82499989, 83045444, 83522716, 83999989, 84477262, 85014193, 85227261
	dc.l	 85909080, 86624989, 87010478, 87499988, 87954534, 88512385, 88977261, 89488625, 89999988, 90430610
	dc.l	 91022715, 91477261, 92045442, 92517470, 93068170, 93489292, 93963056, 94499988, 95020649, 95454533
	dc.l	 96016030, 96409078, 97045442, 97542601, 98024463, 98542768, 99034078, 99431805,100227260

*****************************************************************
GetCompatibleFormats:
; a0: struct BoardInfo *bi
; d7: RGBFTYPE RGBFormat
; for the CirrusGD542X, it is pretty easy: planar modes are compatible to planar only,
; all other formats can coexist without any problems

	cmp.l	#RGBFB_PLANAR,d7
	beq	.planar

.chunky:
	move.l	#~RGBFF_PLANAR,d0
	bra	.end

.planar:
	move.l	#RGBFF_PLANAR,d0
.end:
	rts

*****************************************************************
EndCode:
   END
