*******************************************************************************************
*  sublibrary to picasso96
*******************************************************************************************

	MACHINE	68020

	NOLIST

	incdir	include:
	include	lvo/exec.i
	include	lvo/expansion.i
	include	exec/libraries.i
	include	exec/resident.i
	include	exec/initializers.i
	include	exec/alerts.i
	include	exec/ables.i
	include	libraries/configvars.i
	include	hardware/intbits.i

	incdir	/privateinclude
	include	boardinfo.i
	include	settings.i

	include	macros.i
	include	CirrusGD542X.i

	include	PicassoII.card_rev.i


gbi_SerialNumber	EQU	gbi_CardData+0*4


	LIST

NAME		MACRO
		dc.b	'PicassoII'
		ENDM

;-----------------------------------------------

Start:						; just in case we are executed
	moveq	#-1,d0
	rts

;-----------------------------------------------

RomTagPri	EQU	0

RomTag:
	dc.w	RTC_MATCHWORD
	dc.l	RomTag
	dc.l	EndCode
	dc.b	RTF_AUTOINIT
	dc.b	VERSION
	dc.b	NT_LIBRARY
	dc.b	RomTagPri
	dc.l	LibName
	dc.l	IDString
	dc.l	InitTable

;-----------------------------------------------

CardName:		NAME
		dc.b	0
LibName:		NAME
		dc.b	'.card',0
IDString:		VSTRING
VERString:		VERSTAG
ExpansionName:	dc.b	'expansion.library',0
ChipName:	dc.b	'picasso96/'
		CIRRUSNAME
		dc.b	'.chip',0
		even

;-----------------------------------------------

InitTable:
	dc.l	card_SIZEOF			; size of library base data space
	dc.l	funcTable			; pointer to function initializers
	dc.l	dataTable			; pointer to data initializers
	dc.l	initRoutine			; routine to run

;-----------------------------------------------

funcTable:
	dc.l	Open
	dc.l	Close
	dc.l	Expunge
	dc.l	Null

	dc.l	FindCard
	dc.l	InitCard

	dc.l	-1

;-----------------------------------------------

dataTable:
	INITBYTE	LN_TYPE,NT_LIBRARY
	INITBYTE	LN_PRI,-50
	INITLONG	LN_NAME,LibName
	INITBYTE	LIB_FLAGS,LIBF_SUMUSED!LIBF_CHANGED
	INITWORD	LIB_VERSION,VERSION
	INITWORD	LIB_REVISION,REVISION
	INITLONG	LIB_IDSTRING,IDString
	INITLONG	card_Name,CardName
	dc.l		0

;-----------------------------------------------

initRoutine:					; d0.l: library base, a0: segment list
	PUSH	a5

	move.l	d0,a5
	move.l	a6,card_ExecBase(a5)
	move.l	a0,card_SegmentList(a5)

	lea	ExpansionName(pc),a1
	moveq	#0,d0
	CALL	OpenLibrary
	move.l	d0,card_ExpansionBase(a5)
	bne	.got_expansion

	ALERT	AG_OpenLib!AO_ExpansionLib

.got_expansion:
	move.l	a5,d0
	POP	a5
	rts

*****************************************************************
Open:
* a6:	library
* d0:	version
*****************************************************************
	addq.w	#1,LIB_OPENCNT(a6)		; mark us as having another opener
	bclr	#LIBB_DELEXP,card_Flags(a6)	; prevent delayed expunges
	move.l	a6,d0
	rts

*****************************************************************
Close:
* a6:	library
*****************************************************************
	moveq	#0,d0			; set the return value
	subq.w	#1,LIB_OPENCNT(a6)		; mark us as having one fewer openers
	bne	.end			; see if there is anyone left with us open

	btst	#LIBB_DELEXP,card_Flags(a6)	; see if we have a delayed expunge pending
	beq	.end
	bsr	Expunge
.end:
	rts

*****************************************************************
Expunge:
* a6:	library
*****************************************************************
	PUSH	d2/a5/a6
	move.l	a6,a5
	move.l	card_ExecBase(a5),a6

	tst.w	LIB_OPENCNT(a5)			; see if anyone has us open
	beq	.not_open

	bset	#LIBB_DELEXP,card_Flags(a5)	; it is still open.  set the delayed expunge flag
	moveq	#0,d0
	bra	.end

.not_open:
	move.l	card_SegmentList(a5),d2		; go ahead and get rid of us.  Store our seglist in d2

	move.l	a5,a1				; unlink from library list
	CALL	Remove

	move.l	card_ExpansionBase(a5),a1
	CALL	CloseLibrary

	moveq	#0,d0				; free our memory
	move.l	a5,a1
	move.w	LIB_NEGSIZE(a5),d0
	sub.l	d0,a1
	add.w	LIB_POSSIZE(a5),d0
	CALL	FreeMem

	move.l	d2,d0				; set up our return value

.end:
	POP	d2/a5/a6
	rts

*****************************************************************
Null:
*****************************************************************
	moveq	#0,d0
	rts

*****************************************************************

MANUFACTURER	EQU	2167

FindCard:					; a0: BoardInfo
	PUSH	a2-a3/a6

	move.l	a0,a2
	move.l	card_ExpansionBase(a6),a6

	suba.l	a0,a0
.find_memory_loop:
	move.l	#MANUFACTURER,d0
	moveq	#11,d1
	CALL	FindConfigDev
	tst.l	d0
	beq	.no_memory_found
	move.l	d0,a0
	bclr	#CDB_CONFIGME,cd_Flags(a0)
	beq	.find_memory_loop
	move.l	cd_BoardAddr(a0),gbi_MemoryBase(a2)

	suba.l	a0,a0
.find_registers_loop:
	move.l	#MANUFACTURER,d0
	moveq	#12,d1
	CALL	FindConfigDev
	tst.l	d0
	beq	.no_registers_found
	move.l	d0,a0
	bclr	#CDB_CONFIGME,cd_Flags(a0)
	beq	.find_registers_loop
	move.l	cd_BoardAddr(a0),gbi_RegisterBase(a2)
	move.l	cd_Rom+er_SerialNumber(a0),gbi_SerialNumber(a2)

	moveq	#-1,d0
.no_registers_found:
.no_memory_found:
	POP	a2-a3/a6
	rts

*****************************************************************
InitCard:
* a0:	struct BoardInfo
* new for Picasso96 1.26:
* a1:	char **ToolTypes
*****************************************************************
	PUSH	a2/a5-a6
	move.l	card_ExecBase(a6),a5

	move.l	a0,a2

	lea	ChipName(pc),a1
	moveq	#0,d0
	move.l	a5,a6
	CALL	OpenLibrary
	move.l	d0,gbi_ChipBase(a2)
	beq	.end

	cmp.l	#$10000,gbi_SerialNumber(a2)
	blt	.no_hardware_irq
	lea	(gbi_HardInterrupt,a2),a1	; no interrupt needed by now !!!
	lea	(InterruptCode,pc),a0
	move.l	a0,(IS_CODE,a1)
	moveq	#INTB_PORTS,d0
	move.l	a5,a6
	CALL	AddIntServer

	ori.l	#BIF_VBLANKINTERRUPT,gbi_Flags(a2)

	lea	SetInterrupt(pc),a1
	move.l	a1,gbi_SetInterrupt(a2)
.no_hardware_irq:

	move.l	gbi_ChipBase(a2),a6
	move.l	a2,a0
	CALL	InitChip
	move.l	a5,a6

	lea	CardName(pc),a1
	move.l	a1,gbi_BoardName(a2)
	move.l	#BT_PicassoII,gbi_BoardType(a2)
	lea	SetSwitch(pc),a1
	move.l	a1,gbi_SetSwitch(a2)
	lea	SetColorArray(pc),a1
	move.l	a1,gbi_SetColorArray(a2)
	lea	SetSpriteColor(pc),a1
	move.l	a1,gbi_SetSpriteColor(a2)

	move.l	(gbi_MemoryBase,a2),(gbi_MemorySpaceBase,a2)
	ori.l	#BIF_CACHEMODECHANGE,gbi_Flags(a2)

	move.l	gbi_RegisterBase(a2),a0
	move.l	gbi_MemoryBase(a2),d0
	swap	d0
	and.b	#$10,d0
	or.b	#$21,d0
	SetTS	d0,TS_ExtendedSequencerMode
	SetTS	#$dd,TS_PerformanceTuning
	SetTS	#$1c,TS_MCLKSelect		; $20

	moveq	#16,d0				; check if we can run with 256x16
	bsr	.set_granularity

	movea.l	gbi_MemoryBase(a2),a0
	move.l	#$a5a55a5a,(a0)			; write high
	CALL	CacheClearU
	movea.l	gbi_MemoryBase(a2),a0
	cmp.l	#$a5a55a5a,(a0)			; read high
	bne	.no_high_memory

	moveq	#0,d0
	bsr	.set_granularity

	move.l	#$200000-RESERVEDMEMORY,gbi_MemorySize(a2)		; 2 MB available
	move.l	#$200000,(gbi_MemorySpaceSize,a2)
	bra	.done

.no_high_memory:

	move.l	gbi_RegisterBase(a2),a0
	SetTS	#$30,TS_DRAMControl

	moveq	#0,d0
	bsr	.set_granularity

	move.l	gbi_MemoryBase(a2),a0
	move.l	#$a5a55a5a,(a0)
	CALL	CacheClearU

	moveq	#16,d0
	bsr	.set_granularity

	move.l	gbi_MemoryBase(a2),a0
	move.l	#$5a5aa5a5,(a0)
	CALL	CacheClearU

	moveq	#0,d0
	bsr	.set_granularity

	move.l	gbi_MemoryBase(a2),a0
	cmp.l	#$5a5aa5a5,(a0)
	beq	.overwritten
	cmp.l	#$a5a55a5a,(a0)
	beq	.correct

	moveq	#0,d0
	bra	.end

.overwritten:
	move.l	#$100000-RESERVEDMEMORY,gbi_MemorySize(a2)		; 1 MB available (minus 2 sprite buffers
	move.l	#$100000,(gbi_MemorySpaceSize,a2)
	bra	.done						;                 and a 64 bytes pattern)

.correct:
	move.l	#$200000-RESERVEDMEMORY,gbi_MemorySize(a2)		; 2 MB available
	move.l	#$200000,(gbi_MemorySpaceSize,a2)

.done:
	moveq	#-1,d0
.end:
	POP	a2/a5-a6
	rts

.set_granularity:
	move.l	gbi_RegisterBase(a2),a0
	move.l	d0,d1
	lsl.l	#2,d1
	and.l	#$7C,d1
	move.l	gbi_MemoryBase(a2),d0
	lsr.l	#8,d0
	lsr.l	#6,d0
	and.l	#$7C,d0
	sub.l	d0,d1
	and.l	#$7C,d1
	move.l	d1,d0

	SetGDC	d0,GDC_Offset0
	nop
	rts

*****************************************************************
SetSwitch:
* a0:	struct BoardInfo
* d0.w:	BOOL state
*****************************************************************
	move.w	gbi_MoniSwitch(a0),d1
	and.w	#~1,d1
	tst.b	d0
	beq	.off
	or.w	#1,d1
.off:
	move.w	gbi_MoniSwitch(a0),d0
	cmp.w	d0,d1
	beq	.no_change

	move.w	d1,gbi_MoniSwitch(a0)
	and.w	#1,d1
	eor.w	#1,d1
	or.w	#%0010,d1
	ror.w	#4,d1
	move.l	gbi_RegisterBase(a0),a0
	add.l	#$8000,a0
	move.b	#0,(a0,d1.w)

.no_change:
	and.w	#1,d0
	rts

*****************************************************************
SetColorArray:
* a0:	struct BoardInfo
* d0.w:	startindex
* d1.w:	count
*****************************************************************
	PUSH	d2/a6
	move.l	4,a6

	lea	gbi_CLUT(a0),a1
	moveq	#8,d2
	sub.w	gbi_BitsPerCannon(a0),d2
	move.l	gbi_RegisterBase(a0),a0

	DISABLE
	move.b	d0,PALETTEADDRESS(a0)

	add.w	d0,a1
	add.w	d0,a1
	add.w	d0,a1

	bra	.start

.loop:
	move.b	(a1)+,d0
	lsr.b	d2,d0
	move.b	d0,$13C8(a0)	; PALETTEDATA
	nop
	move.b	(a1)+,d0
	lsr.b	d2,d0
	move.b	d0,$13C8(a0)	; PALETTEDATA
	nop
	move.b	(a1)+,d0
	lsr.b	d2,d0
	move.b	d0,$13C8(a0)	; PALETTEDATA
	nop
.start:
	dbra	d1,.loop

	ENABLE
	POP	d2/a6
	rts

*****************************************************************
SetSpriteColor:
* set sprite colors
* a0: 	struct BoardInfo
* d0.b:	Index
* d1.b:	Red
* d2.b:	Green
* d3.b:	Blue
* d7:	RGBFormat
*****************************************************************
	tst.b	d0
	beq	.color_0
	cmp.b	#2,d0
	bne	.end
.color_1:
	moveq	#15,d0
.color_0:
	move.l	gbi_RegisterBase(a0),a0

	lsr.b	#2,d1
	lsr.b	#2,d2
	lsr.b	#2,d3
	move.b	d0,$3C8(a0)

	move.b	#TS_GraphicsCursorAttributes,TSI(a0)
	move.b	TSD(a0),d0
	or.b	#2,d0				; reveal extended DAC colors
	move.b	d0,TSD(a0)

	DELAY
	move.b	d1,$13C8(a0)
	nop
	DELAY
	move.b	d2,$13C8(a0)
	nop
	DELAY
	move.b	d3,$13C8(a0)
	nop

	and.b	#~2,d0				; hide extended DAC colors
	move.b	d0,TSD(a0)

.end:
	rts


*****************************************************************
SetInterrupt:
* a0: 	struct BoardInfo
* d0.w:	state
*****************************************************************
	move.l	gbi_RegisterBase(a0),a0
	move.b	#CRTC_VerticalSyncEnd,(CRTCI,a0)
	move.b	(CRTCD,a0),d1
	or.b	#(1<<5)|(1<<4),d1
	tst.b	d0
	beq	.off
.on:
	and.b	#~((1<<5)|(1<<4)),d1
	move.b	d1,(CRTCD,a0)
	or.b	#(1<<4),d1
	move.b	d1,(CRTCD,a0)
	move.b	#0,($1001,a0)
	rts
.off:
	move.b	d1,(CRTCD,a0)
	move.w	#0,($1000,a0)
	rts

InterruptCode:
	moveq	#0,d0
	move.l	(gbi_RegisterBase,a1),a6
	move.b	(INPUTSTATUS0,a6),d0
	and.b	#(1<<7),d0				; CRT Interrupt ?
	beq	.no_interrupt_pending
.interrupt_pending:
	move.b	(CRTCI,a6),d1
	swap	d1
	move.b	#CRTC_VerticalSyncEnd,(CRTCI,a6)
	move.b	(CRTCD,a6),d1
	and.b	#~(1<<4),d1				; ClearVerticalInterrupt
	move.b	d1,(CRTCD,a6)
;	or.b	#(1<<4),d1
;	move.b	d1,(CRTCD,a6)
	swap	d1
	move.b	d1,(CRTCI,a6)

	move.l	(gbi_ExecBase,a1),a6
	lea	(gbi_SoftInterrupt,a1),a1
	CALL	Cause

	moveq	#1,d0
.no_interrupt_pending:
	rts

*****************************************************************
EndCode:
   END
