#ifndef boardinfo_H
#define boardinfo_H

#ifndef  LIBRARIES_PICASSO96_H
#include	<libraries/Picasso96.h>
#endif

#ifndef  EXEC_INTERRUPTS_H
#include	<exec/interrupts.h>
#endif

#ifndef  EXEC_SEMAPHORES_H
#include	<exec/semaphores.h>
#endif

#ifndef  GRAPHICS_GFX_H
#include	<graphics/gfx.h>
#endif

#define DI_P96_INVALID  0x1000
#define DI_P96_MONITOOL	0x2000

/************************************************************************/
/* Types for BoardType Identification
 */
typedef enum {
	BT_NoBoard,
	BT_oMniBus,
	BT_Graffity,
	BT_CyberVision,
	BT_Domino,
	BT_Merlin,
	BT_PicassoII,
	BT_Piccolo,
	BT_RetinaBLT,
	BT_Spectrum,
	BT_PicassoIV,
	BT_PiccoloSD64,
	BT_A2410,
	BT_Pixel64,
	BT_uaegfx,
	BT_CVision3D,
	BT_Prototype1,
	BT_Prototype2,
	BT_Prototype3,
	BT_Prototype4,
	BT_Prototype5,
	BT_MaxBoardTypes
} BTYPE;

/************************************************************************/
/* Types for PaletteChipType Identification
 */
typedef enum {
	PCT_Unknown,
	PCT_S11483,				// Sierra S11483: HiColor 15 bit, oMniBus, Domino
	PCT_S15025,				// Sierra S15025: TrueColor 32 bit, oMniBus
	PCT_CirrusGD542x,		// Cirrus GD542x internal: TrueColor 24 bit
	PCT_Domino,				// is in fact a Sierra S11483
	PCT_BT482,				// BrookTree BT482: TrueColor 32 bit, Merlin
	PCT_Music,				// Music MU9C4910: TrueColor 24 bit, oMniBus
	PCT_ICS5300,			// ICS 5300: ...., Retina BLT Z3
	PCT_CirrusGD5446,		// Cirrus GD5446 internal: TrueColor 24 bit
	PCT_CirrusGD5434,		// Cirrus GD5434 internal: TrueColor 32 bit
	PCT_S3Trio64,			// S3 Trio64 internal: TrueColor 32 bit
	PCT_A2410_xxx,			// A2410 DAC, *type unknown*
	PCT_S3ViRGE,			// S3 ViRGE internal: TrueColor 32 bit
	PCT_MaxPaletteChipTypes
} PCTYPE;

/************************************************************************/
/* Types for GraphicsControllerType Identification
 */
typedef enum {
	GCT_Unknown,
	GCT_ET4000,
	GCT_ETW32,	
	GCT_CirrusGD542x,
	GCT_NCR77C32BLT,	
	GCT_CirrusGD5446,
	GCT_CirrusGD5434,
	GCT_S3Trio64,
	GCT_TI34010,
	GCT_S3ViRGE,
	GCT_MaxGraphicsControllerTypes
} GCTYPE;

/************************************************************************/

#define	RGBFF_PLANAR	RGBFF_NONE
#define	RGBFF_CHUNKY	RGBFF_CLUT

#define	RGBFB_PLANAR	RGBFB_NONE
#define	RGBFB_CHUNKY	RGBFB_CLUT

/************************************************************************/

enum{
	DPMS_ON,			/* Full operation                             */
	DPMS_STANDBY,	/* Optional state of minimal power reduction  */
	DPMS_SUSPEND,	/* Significant reduction of power consumption */
	DPMS_OFF			/* Lowest level of power consumption          */
};

/************************************************************************/

enum{
	PLANAR,
	CHUNKY,
	HICOLOR,
	TRUECOLOR,
	TRUEALPHA,
	MAXMODES
};

/************************************************************************/

struct CLUTEntry {
	UBYTE	Red;
	UBYTE	Green;
	UBYTE	Blue;
};

/************************************************************************/

struct GfxMemChunk {
	struct MinNode Node;
	char *Ptr;
	ULONG Size;
	BOOL Used;
};

/************************************************************************/

struct Template {
	APTR			Memory;
	WORD			BytesPerRow;
	UBYTE			XOffset;				// 0 <= XOffset <= 15
	UBYTE			DrawMode;
	ULONG			FgPen;
	ULONG			BgPen;
};

/************************************************************************/

struct Pattern {
	APTR			Memory;
	UWORD			XOffset, YOffset;
	ULONG			FgPen, BgPen;
	UBYTE			Size;					// Width: 16, Height: (1<<pat_Size)
	UBYTE			DrawMode;
};

/************************************************************************/

struct BitMapExtra {
	struct MinNode			BoardNode;
	struct BitMapExtra	*HashChain;
	APTR						Match;
	struct BitMap			*BitMap;
	struct BoardInfo		*BoardInfo;
	APTR						MemChunk;
	struct RenderInfo		RenderInfo;
	UWORD						Width, Height;
	UWORD						Flags;
};

/* BitMapExtra flags */
#define	BMEF_ONBOARD			0x0001
#define	BMEF_SPECIAL			0x0002
#define  BMEF_VISIBLE			0x0800
#define  BMEF_DISPLAYABLE		0x1000
#define	BMEF_SPRITESAVED		0x2000
#define	BMEF_CHECKSPRITE		0x4000
#define	BMEF_INUSE				0x8000

/************************************************************************/

struct SpecialFeature {
	struct MinNode		Node;
	struct BoardInfo	*BoardInfo;
	struct BitMap		*BitMap;
	ULONG					Type;
	APTR					FeatureData;
};

enum  {
	SFT_INVALID, SFT_FLICKERFIXER, SFT_VIDEOCAPTURE, SFT_VIDEOWINDOW, SFT_MEMORYWINDOW
};

#define	FA_Restore			(TAG_USER+0)			/* becomes visible again */
#define	FA_Onboard			(TAG_USER+1)
#define	FA_Active			(TAG_USER+2)
#define	FA_Left				(TAG_USER+3)
#define	FA_Top				(TAG_USER+4)
#define	FA_Width				(TAG_USER+5)
#define	FA_Height			(TAG_USER+6)
#define	FA_Format			(TAG_USER+7)
#define	FA_Color				(TAG_USER+8)
#define	FA_Occlusion		(TAG_USER+9)
#define	FA_SourceWidth		(TAG_USER+10)
#define	FA_SourceHeight	(TAG_USER+11)
#define	FA_MinWidth			(TAG_USER+12)
#define	FA_MinHeight		(TAG_USER+13)
#define	FA_MaxWidth			(TAG_USER+14)
#define	FA_MaxHeight		(TAG_USER+15)
#define	FA_Interlace		(TAG_USER+16)
#define	FA_PAL				(TAG_USER+17)
#define	FA_BitMap			(TAG_USER+18)
#define	FA_Brightness		(TAG_USER+19)
#define	FA_ModeInfo			(TAG_USER+20)
#define	FA_ModeFormat		(TAG_USER+21)

/************************************************************************/

struct BoardInfo{
	UBYTE 								*RegisterBase, *MemoryBase, *MemoryIOBase;
	ULONG									MemorySize;
	char									*BoardName,VBIName[32];
	struct CardBase					*CardBase;
	struct ChipBase					*ChipBase;
	struct ExecBase					*ExecBase;
	struct Library						*UtilBase;
	struct Interrupt					HardInterrupt;
	struct Interrupt					SoftInterrupt;
	struct SignalSemaphore			BoardLock;
	struct MinList						ResolutionsList;
	BTYPE									BoardType;
	PCTYPE								PaletteChipType;
	GCTYPE								GraphicsControllerType;
	UWORD									MoniSwitch;
	UWORD									BitsPerCannon;
	ULONG									Flags;
	UWORD									SoftSpriteFlags;
	UWORD									ChipFlags;					// private, chip specific, not touched by RTG
	ULONG									CardFlags;					// private, card specific, not touched by RTG

	UWORD									BoardNum;
	UWORD									RGBFormats;

	UWORD									MaxHorValue[MAXMODES];
	UWORD									MaxVerValue[MAXMODES];
	UWORD									MaxHorResolution[MAXMODES];
	UWORD									MaxVerResolution[MAXMODES];
	ULONG									MaxMemorySize, MaxChunkSize;

	ULONG									__obsolete;

	ULONG									PixelClockCount[MAXMODES];

	APTR __asm							(*AllocCardMem)(register __a0 struct BoardInfo *bi, register __d0 ULONG size, register __d1 BOOL force, register __d2 BOOL system);
	BOOL __asm							(*FreeCardMem)(register __a0 struct BoardInfo *bi, register __a1 APTR membase);

	BOOL __asm							(*SetSwitch)(register __a0 struct BoardInfo *, register __d0 BOOL);

	void __asm							(*SetColorArray)(register __a0 struct BoardInfo *, register __d0 UWORD, register __d1 UWORD);

	void __asm							(*SetDAC)(register __a0 struct BoardInfo *, register __d7 RGBFTYPE);
	void __asm							(*SetGC)(register __a0 struct BoardInfo *, register __a1 struct ModeInfo *, register __d0 BOOL);
	void __asm							(*SetPanning)(register __a0 struct BoardInfo *, register __a1 UBYTE *, register __d0 UWORD, register __d1 WORD, register __d2 WORD, register __d7 RGBFTYPE);
	UWORD __asm							(*CalculateBytesPerRow)(register __a0 struct BoardInfo *, register __d0 UWORD, register __d7 RGBFTYPE);
	APTR __asm							(*CalculateMemory)(register __a0 struct BoardInfo *, register __a1 APTR, register __d7 RGBFTYPE);
	ULONG __asm							(*GetCompatibleFormats)(register __a0 struct BoardInfo *, register __d7 RGBFTYPE);
	BOOL __asm							(*SetDisplay)(register __a0 struct BoardInfo *, register __d0 BOOL);

	LONG __asm							(*ResolvePixelClock)(register __a0 struct BoardInfo *, register __a1 struct ModeInfo *, register __d0 ULONG, register __d7 RGBFTYPE);
	ULONG	__asm							(*GetPixelClock)(register __a0 struct BoardInfo *bi, register __a1 struct ModeInfo *mi, register __d0 Index, register __d7 RGBFormat);
	void __asm							(*SetClock)(register __a0 struct BoardInfo *);

	void __asm							(*SetMemoryMode)(register __a0 struct BoardInfo *, register __d7 RGBFTYPE);
	void __asm							(*SetWriteMask)(register __a0 struct BoardInfo *, register __d0 UBYTE);
	void __asm							(*SetClearMask)(register __a0 struct BoardInfo *, register __d0 UBYTE);
	void __asm							(*SetReadPlane)(register __a0 struct BoardInfo *, register __d0 UBYTE);

	void __asm							(*WaitVerticalSync)(register __a0 struct BoardInfo *, register __d0 BOOL);
	BOOL __asm							(*SetInterrupt)(register __a0 struct BoardInfo *, register __d0 BOOL);

	void __asm							(*WaitBlitter)(register __a0 struct BoardInfo *);

	void __asm							(*ScrollPlanar)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __d0 UWORD, register __d1 UWORD, register __d2 UWORD, register __d3 UWORD, register __d4 UWORD, register __d5 UWORD, register __d6 UBYTE);
	void __asm							(*ScrollPlanarDefault)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __d0 UWORD, register __d1 UWORD, register __d2 UWORD, register __d3 UWORD, register __d4 UWORD, register __d5 UWORD, register __d6 UBYTE);
	void __asm							(*UpdatePlanar)(register __a0 struct BoardInfo *, register __a1 struct BitMap *, register __a2 struct RenderInfo *, register __d0 SHORT, register __d1 SHORT, register __d2 SHORT, register __d3 SHORT, register __d4 UBYTE);
	void __asm							(*UpdatePlanarDefault)(register __a0 struct BoardInfo *, register __a1 struct BitMap *, register __a2 struct RenderInfo *, register __d0 SHORT, register __d1 SHORT, register __d2 SHORT, register __d3 SHORT, register __d4 UBYTE);
	void __asm							(*BlitPlanar2Chunky)(register __a0 struct BoardInfo *, register __a1 struct BitMap *, register __a2 struct RenderInfo *, register __d0 SHORT, register __d1 SHORT, register __d2 SHORT, register __d3 SHORT, register __d4 SHORT, register __d5 SHORT, register __d6 UBYTE, register __d7 UBYTE);
	void __asm							(*BlitPlanar2ChunkyDefault)(register __a0 struct BoardInfo *, register __a1 struct BitMap *, register __a2 struct RenderInfo *, register __d0 SHORT, register __d1 SHORT, register __d2 SHORT, register __d3 SHORT, register __d4 SHORT, register __d5 SHORT, register __d6 UBYTE, register __d7 UBYTE);

	void __asm							(*FillRect)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 ULONG, register __d5 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*FillRectDefault)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 ULONG, register __d5 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*InvertRect)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*InvertRectDefault)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*BlitRect)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 WORD, register __d5 WORD, register __d6 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*BlitRectDefault)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 WORD, register __d5 WORD, register __d6 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*BlitTemplate)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __a2 struct Template *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*BlitTemplateDefault)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __a2 struct Template *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*BlitPattern)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __a2 struct Pattern *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*BlitPatternDefault)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __a2 struct Pattern *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*DrawLine)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*DrawLineDefault)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*BlitRectNoMaskComplete)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __a2 struct RenderInfo *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 WORD, register __d5 WORD, register __d6 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*BlitRectNoMaskCompleteDefault)(register __a0 struct BoardInfo *, register __a1 struct RenderInfo *, register __a2 struct RenderInfo *, register __d0 WORD, register __d1 WORD, register __d2 WORD, register __d3 WORD, register __d4 WORD, register __d5 WORD, register __d6 UBYTE, register __d7 RGBFTYPE);
	void __asm							(*Reserved0)(register __a0 struct BoardInfo *);
	void __asm							(*Reserved0Default)(register __a0 struct BoardInfo *);
	void __asm							(*Reserved1)(register __a0 struct BoardInfo *);
	void __asm							(*Reserved1Default)(register __a0 struct BoardInfo *);
	void __asm							(*Reserved2)(register __a0 struct BoardInfo *);
	void __asm							(*Reserved2Default)(register __a0 struct BoardInfo *);
	void __asm							(*Reserved3)(register __a0 struct BoardInfo *);
	void __asm							(*Reserved3Default)(register __a0 struct BoardInfo *);
	void __asm							(*Reserved4)(register __a0 struct BoardInfo *);
	void __asm							(*Reserved4Default)(register __a0 struct BoardInfo *);
	void __asm							(*Reserved5)(register __a0 struct BoardInfo *);
	void __asm							(*Reserved5Default)(register __a0 struct BoardInfo *);
	void __asm							(*Reserved6)(register __a0 struct BoardInfo *);
	void __asm							(*Reserved6Default)(register __a0 struct BoardInfo *);
	void __asm							(*SetDPMSLevel)(register __a0 struct BoardInfo *, register __d0 ULONG DPMSLevel);
	void __asm							(*ResetChip)(register __a0 struct BoardInfo *);
	ULONG __asm							(*GetFeatureAttrs)(register __a0 struct BoardInfo *bi, register __a1 APTR FeatureData, register __d0 ULONG Type, register __a2 struct TagItem *Tags);

	struct BitMap * __asm 			(*AllocBitMap)(register __a0 struct BoardInfo *, register __d0 ULONG, register __d1 ULONG, register __a1 struct TagItem *);
	BOOL __asm							(*FreeBitMap)(register __a0 struct BoardInfo *, register __a1 struct BitMap *, register __a2 struct TagItem *);
	ULONG __asm							(*GetBitMapAttr)(register __a0 struct BoardInfo *, register __a1 struct BitMap *, register __d0 ULONG);

	BOOL __asm							(*SetSprite)(register __a0 struct BoardInfo *, register __d0 BOOL, register __d7 RGBFTYPE);
	void __asm							(*SetSpritePosition)(register __a0 struct BoardInfo *, register __d0 WORD, register __d1 WORD, register __d7 RGBFTYPE);
	void __asm							(*SetSpriteImage)(register __a0 struct BoardInfo *, register __d7 RGBFTYPE);
	void __asm							(*SetSpriteColor)(register __a0 struct BoardInfo *, register __d0 UBYTE, register __d1 UBYTE, register __d2 UBYTE, register __d3 UBYTE, register __d7 RGBFTYPE);

	APTR __asm							(*CreateFeature)(register __a0 struct BoardInfo *bi, register __d0 ULONG Type, register __a1 struct TagItem *Tags);
	ULONG __asm							(*SetFeatureAttrs)(register __a0 struct BoardInfo *bi, register __a1 APTR FeatureData, register __d0 ULONG Type, register __a2 struct TagItem *Tags);
	BOOL __asm							(*DeleteFeature)(register __a0 struct BoardInfo *bi, register __a1 APTR FeatureData, register __d0 ULONG Type);
	struct MinList						SpecialFeatures;

	struct ModeInfo					*ModeInfo;								/* Chip Settings Stuff */
	RGBFTYPE								RGBFormat;
	WORD									XOffset, YOffset;
	UBYTE									Depth;
	UBYTE									ClearMask;
	BOOL									Border;
	ULONG									Mask;
	struct CLUTEntry					CLUT[256];

	struct ViewPort					*ViewPort;								/* ViewPort Stuff */
	struct BitMap						*VisibleBitMap;
	struct BitMapExtra				*BitMapExtra;
	struct MinList						BitMapList;
	struct MinList						MemList;

	WORD									MouseX, MouseY;						/* Sprite Stuff */
	UBYTE									MouseWidth, MouseHeight;
	UBYTE									MouseXOffset, MouseYOffset;
	UWORD									*MouseImage;
	UBYTE									MousePens[4];
	struct Rectangle					MouseRect;
	UBYTE									*MouseChunky;
	UWORD									*MouseRendered;
	UBYTE									*MouseSaveBuffer;

	ULONG									ChipData[16];							/* for chip driver needs */
	ULONG									CardData[16];							/* for card driver needs */
	
	APTR									MemorySpaceBase;						/* the base address of the board memory address space */
	ULONG									MemorySpaceSize;						/* size of that area */

	APTR									DoubleBufferList;						/* chain of dbinfos being notified on vblanks */
	
	struct timeval						SyncTime;								/* system time when screen was set up, used for pseudo vblanks */
	ULONG									SyncPeriod;								/* length of one frame in micros */
	struct MsgPort						SoftVBlankPort;						/* MsgPort for software emulation of board interrupt */
};

/* BoardInfo flags */
/*  0-15: hardware flags */
/* 16-31: user flags */
#define BIB_HARDWARESPRITE		0		/* board has hardware sprite */
#define BIB_NOMEMORYMODEMIX	1		/* board does not support modifying planar bitmaps while displaying chunky and vice versa */
#define BIB_NEEDSALIGNMENT		2		/* bitmaps have to be aligned (not yet supported!) */
#define BIB_CACHEMODECHANGE	3		/* board memory may be set to Imprecise (060) or Nonserialised (040) */
#define BIB_VBLANKINTERRUPT	4		/* board can cause a hardware interrupt on a vertical retrace */

#define BIB_DBLSCANDBLSPRITEY	8		/* hardware sprite y position is doubled on doublescan display modes */
#define BIB_ILACEHALFSPRITEY	9		/* hardware sprite y position is halved on interlace display modes */
#define BIB_ILACEDBLROWOFFSET	10		/* doubled row offset in interlaced display modes needs additional horizontal bit */

#define BIB_FLICKERFIXER		12		/* board can flicker fix Amiga RGB signal */
#define BIB_VIDEOCAPTURE		13		/* board can capture video data to a memory area */
#define BIB_VIDEOWINDOW			14		/* board can display a second mem area as a pip */
#define BIB_BLITTER				15		/* board has blitter */

#define BIB_HIRESSPRITE			16		/* mouse sprite has double resolution */
#define BIB_BIGSPRITE			17		/* user wants big mouse sprite */
#define BIB_BORDEROVERRIDE		18		/* user wants to override system overscan border prefs */
#define BIB_BORDERBLANK			19		/* user wants border blanking */
#define BIB_INDISPLAYCHAIN		20		/* board switches Amiga signal */
#define BIB_QUIET					21		/* not yet implemented */
#define BIB_NOMASKBLITS			22		/* perform blits without taking care of mask */
#define BIB_NOC2PBLITS			23		/* use CPU for planar to chunky conversions */
#define BIB_NOBLITTER			24		/* disable all blitter functions */

#define BIB_OVERCLOCK			31		/* enable overclocking for some boards */

#define BIB_IGNOREMASK	BIB_NOMASKBLITS

#define BIF_HARDWARESPRITE			(1<<BIB_HARDWARESPRITE)
#define BIF_NOMEMORYMODEMIX		(1<<BIB_NOMEMORYMODEMIX)
#define BIF_NEEDSALIGNMENT			(1<<BIB_NEEDSALIGNMENT)
#define BIF_CACHEMODECHANGE		(1<<BIB_CACHEMODECHANGE)
#define BIF_VBLANKINTERRUPT		(1<<BIB_VBLANKINTERRUPT)
#define BIF_DBLSCANDBLSPRITEY		(1<<BIB_DBLSCANDBLSPRITEY)
#define BIF_ILACEHALFSPRITEY		(1<<BIB_ILACEHALFSPRITEY)
#define BIF_ILACEDBLROWOFFSET		(1<<BIB_ILACEDBLROWOFFSET)
#define BIF_FLICKERFIXER			(1<<BIB_FLICKERFIXER)
#define BIF_VIDEOCAPTURE			(1<<BIB_VIDEOCAPTURE)
#define BIF_VIDEOWINDOW				(1<<BIB_VIDEOWINDOW)
#define BIF_BLITTER					(1<<BIB_BLITTER)
#define BIF_HIRESSPRITE				(1<<BIB_HIRESSPRITE)
#define BIF_BIGSPRITE				(1<<BIB_BIGSPRITE)
#define BIF_BORDEROVERRIDE			(1<<BIB_BORDEROVERRIDE)
#define BIF_BORDERBLANK				(1<<BIB_BORDERBLANK)
#define BIF_INDISPLAYCHAIN			(1<<BIB_INDISPLAYCHAIN)
#define BIF_QUIET						(1<<BIB_QUIET)
#define BIF_NOMASKBLITS				(1<<BIB_NOMASKBLITS)
#define BIF_NOC2PBLITS				(1<<BIB_NOC2PBLITS)
#define BIF_NOBLITTER				(1<<BIB_NOBLITTER)
#define BIF_OVERCLOCK				(1<<BIB_OVERCLOCK)

#define BIF_IGNOREMASK 	BIF_NOMASKBLITS

/************************************************************************/

struct CardBase {
	struct Library		LibNode;
	UBYTE					Flags;
	UBYTE					pad;

	struct ExecBase	*ExecBase;
	struct Library		*ExpansionBase;

	APTR					SegList;
	char					*Name;
};

struct ChipBase {
	struct Library		LibNode;
	UBYTE					Flags;
	UBYTE					pad;

	struct ExecBase	*ExecBase;

	APTR					SegList;
};

/************************************************************************/
#endif
