#include <intuition/screens.h>
#include <graphics/rastport.h>
#include <graphics/gfxmacros.h>

#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#include <proto/dos.h>
#include <proto/Picasso96.h>

char	ScreenTitle[] = "Picasso96 BlitPattern Test";
WORD	Pens[] = {~0};

#define	PatSize	2
UWORD Muster[(1L<<PatSize)]= { 0x8888, 0x0000, 0x2222, 0x0000 };

void Render(struct RastPort *rp, WORD x, WORD y, UBYTE mask)
{
	rp->Mask = 0xff;

	SetDrMd(rp, JAM2);
	SetAPen(rp, 7);
	SetAfPt(rp, NULL, 0);
	RectFill(rp, x+0*32, y+8, x+31+7*32, y+24-1);

	SetAPen(rp, 1);
	SetBPen(rp, 2);

	SetAfPt(rp, Muster, PatSize);

	rp->Mask = mask;

	SetDrMd(rp, JAM1);
	RectFill(rp, x+0*32, y, x+31+0*32, y+31);
	SetDrMd(rp, JAM1|INVERSVID);
	RectFill(rp, x+1*32, y, x+31+1*32, y+31);
	SetDrMd(rp, JAM1|COMPLEMENT);
	RectFill(rp, x+2*32, y, x+31+2*32, y+31);
	SetDrMd(rp, JAM1|INVERSVID|COMPLEMENT);
	RectFill(rp, x+3*32, y, x+31+3*32, y+31);
	SetDrMd(rp, JAM2);
	RectFill(rp, x+4*32, y, x+31+4*32, y+31);
	SetDrMd(rp, JAM2|INVERSVID);
	RectFill(rp, x+5*32, y, x+31+5*32, y+31);
	SetDrMd(rp, JAM2|COMPLEMENT);
	RectFill(rp, x+6*32, y, x+31+6*32, y+31);
	SetDrMd(rp, JAM2|INVERSVID|COMPLEMENT);
	RectFill(rp, x+7*32, y, x+31+7*32, y+31);

	rp->Mask = 0xff;
}

struct LayerMsg
{
	APTR					Layer;
	struct Rectangle	Bounds;
	LONG					OffsetX,	OffsetY;
};

static void __asm __saveds BackfillRoutine(
	register __a0 struct Hook *Hook,
	register __a2 struct RastPort *RPort,
	register __a1 struct LayerMsg *LayerMsg)
{
	struct RastPort	*rp = (struct RastPort *)Hook->h_Data;
	rp->BitMap = RPort->BitMap;

	RectFill(rp,
		LayerMsg->Bounds.MinX,LayerMsg->Bounds.MinY,
		LayerMsg->Bounds.MaxX,LayerMsg->Bounds.MaxY);
}

struct RastPort	HookRastPort;
struct Hook			BackFillHook = {{NULL, NULL}, (void *)BackfillRoutine, NULL, &HookRastPort};

main(void)
{
	ULONG	DisplayID;

	if(INVALID_ID != (DisplayID = p96RequestModeIDTags(TAG_DONE))){
		struct Screen	*sc;

		if(sc=OpenScreenTags(NULL,
									SA_DisplayID,DisplayID,SA_Depth,8,SA_Width,640,SA_Height,480,SA_Pens,Pens,SA_FullPalette,TRUE,SA_Title,(ULONG)ScreenTitle,TAG_DONE)){
			struct BitMap	*bm;

			/* width of bitmap doesn't allow it to be put on the board */ 

			if(bm = AllocBitMap(4100,32,8,BMF_CLEAR,sc->RastPort.BitMap)){
				struct Window	*bdwd;

				InitRastPort(&HookRastPort);
				SetDrMd(&HookRastPort, JAM2);
				SetAfPt(&HookRastPort, Muster, PatSize);
				SetAPen(&HookRastPort, 1);
				SetBPen(&HookRastPort, 0);

				if(bdwd=OpenWindowTags(NULL,WA_Left,0,WA_Top,0,WA_Width,640,WA_Height,480,WA_BackFill,&BackFillHook,WA_CustomScreen,sc,WA_Borderless,TRUE,WA_Backdrop,TRUE,TAG_DONE)){
					struct Window	*wd;

					if(wd=OpenWindowTags(NULL,WA_Left,0,WA_Width,12*32,WA_Height,7*32,WA_DragBar,TRUE,WA_CloseGadget,TRUE,WA_Activate,TRUE,WA_IDCMP,IDCMP_CLOSEWINDOW|IDCMP_RAWKEY,WA_CustomScreen,sc,WA_Title,(ULONG)ScreenTitle,TAG_DONE)){
						struct Message		*msg;
						struct RastPort	*rp, prp;

						rp = wd->RPort;
						prp = *rp;

						prp.Layer = NULL;
						prp.BitMap = bm;

						Render(&prp, 0, 0, 0x05);
						BltBitMapRastPort(bm, 0, 0, rp, 2*32+12, 2*40+24, 8*32, 32, 0xc0);

						BltBitMap(bm, 0, 0, bm, 0, 0, 8*32, 32, 0x00, 0xff, NULL);
						Render(&prp, 0, 0, 0xff);
						BltBitMapRastPort(bm, 0, 0, rp, 2*32+12, 24, 8*32, 32, 0xc0);

						Render(rp, 2*32+12, 1*40+24, 0xff);

						Render(rp, 2*32+12, 3*40+24, 0x05);

						SetDrMd(rp, JAM1);
						SetAPen(rp, 1);
						SetBPen(rp, 0);

						Move(rp,12,62);
						Text(rp,"No Mask",7);
						Move(rp,12,2*40+62);
						Text(rp,"Mask $5",7);

						Move(rp,12+3*32,7*16+75);
						Text(rp,"INV",3);
						Move(rp,12+5*32,7*16+75);
						Text(rp,"INV",3);
						Move(rp,12+7*32,7*16+75);
						Text(rp,"INV",3);
						Move(rp,12+9*32,7*16+75);
						Text(rp,"INV",3);

						Move(rp,28+4*32,7*16+85);
						Text(rp,"COMP",4);
						Move(rp,28+8*32,7*16+85);
						Text(rp,"COMP",4);

						Move(rp,4*32,7*16+94);
						Text(rp,"JAM1",4);
						Move(rp,8*32,7*16+94);
						Text(rp,"JAM2",4);

						Move(rp,10*32+16,1*40+3);
						Text(rp,"RAM",3);

						Move(rp,10*32+16,2*40+3);
						Text(rp,"CARD",4);

						Move(rp,10*32+16,3*40+3);
						Text(rp,"RAM",3);

						Move(rp,10*32+16,4*40+3);
						Text(rp,"CARD",4);

						WaitPort(wd->UserPort);
						
						Forbid();
						while(msg = GetMsg(wd->UserPort)) ReplyMsg(msg);
						Permit();

						CloseWindow(wd);
					}
					CloseWindow(bdwd);
				}
				FreeBitMap(bm);
			}
			CloseScreen(sc);
		}
	}
}
